@@ -61,6 +61,8 @@
 
 #define NEED_REAL_STDIO 
 
+#include "tk.h"
+#include "tkVMacro.h"
 #include "imgInt.h"
 #include <stdio.h>
 #include <string.h>
