@@ -9,12 +9,12 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkCanvImg.c,v 1.2 1998/09/14 18:23:05 stanton Exp $
+ * SCCS: @(#) tkCanvImg.c 1.18 96/05/03 10:49:09
  */
 
 #include "tkInt.h"
 #include "tkPort.h"
-#include "tkCanvases.h"
+#include "tkCanvas.h"
 
 /*
  * The structure below defines the record for each image item.
@@ -30,49 +30,25 @@ typedef struct ImageItem  {
 				 * (x,y). */
     char *imageString;		/* String describing -image option (malloc-ed).
 				 * NULL means no image right now. */
-    char *activeImageString;	/* String describing -activeimage option.
-				 * NULL means no image right now. */
-    char *disabledImageString;	/* String describing -disabledimage option.
-				 * NULL means no image right now. */
     Tk_Image image;		/* Image to display in window, or NULL if
 				 * no image at present. */
-    Tk_Image activeImage;	/* Image to display in window, or NULL if
-				 * no image at present. */
-    Tk_Image disabledImage;	/* Image to display in window, or NULL if
-				 * no image at present. */
 } ImageItem;
 
 /*
  * Information used for parsing configuration specs:
  */
 
-static Tk_CustomOption stateOption = {
-    Tk_StateParseProc,
-    Tk_StatePrintProc, (ClientData) 2
-};
-static Tk_CustomOption tagsOption = {
-    Tk_CanvasTagsParseProc,
+static Tk_CustomOption tagsOption = {Tk_CanvasTagsParseProc,
     Tk_CanvasTagsPrintProc, (ClientData) NULL
 };
 
 static Tk_ConfigSpec configSpecs[] = {
-    {TK_CONFIG_STRING, "-activeimage", (char *) NULL, (char *) NULL,
-	(char *) NULL, Tk_Offset(ImageItem, activeImageString),
-	TK_CONFIG_NULL_OK},
     {TK_CONFIG_ANCHOR, "-anchor", (char *) NULL, (char *) NULL,
 	"center", Tk_Offset(ImageItem, anchor), TK_CONFIG_DONT_SET_DEFAULT},
-    {TK_CONFIG_OBJECT, "-disabledimage", (char *) NULL, (char *) NULL,
-	(char *) NULL, Tk_Offset(ImageItem, disabledImageString),
-	TK_CONFIG_NULL_OK},
     {TK_CONFIG_OBJECT, "-image", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(ImageItem, imageString), TK_CONFIG_NULL_OK},
-    {TK_CONFIG_CUSTOM, "-state", (char *) NULL, (char *) NULL,
-	(char *) NULL, Tk_Offset(Tk_Item, state), TK_CONFIG_NULL_OK,
-	&stateOption},
     {TK_CONFIG_CUSTOM, "-tags", (char *) NULL, (char *) NULL,
 	(char *) NULL, 0, TK_CONFIG_NULL_OK, &tagsOption},
-    {TK_CONFIG_CALLBACK, "-updatecommand", (char *) NULL, (char *) NULL,
-	(char *) NULL, Tk_Offset(Tk_Item, updateCmd), TK_CONFIG_NULL_OK},
     {TK_CONFIG_END, (char *) NULL, (char *) NULL, (char *) NULL,
 	(char *) NULL, 0, 0}
 };
@@ -91,8 +67,6 @@ static int		ImageToArea _ANSI_ARGS_((Tk_Canvas canvas,
 			    Tk_Item *itemPtr, double *rectPtr));
 static double		ImageToPoint _ANSI_ARGS_((Tk_Canvas canvas,
 			    Tk_Item *itemPtr, double *coordPtr));
-static int		ImageToPostscript _ANSI_ARGS_((Tcl_Interp *interp,
-			    Tk_Canvas canvas, Tk_Item *itemPtr, int prepass));
 static void		ComputeImageBbox _ANSI_ARGS_((Tk_Canvas canvas,
 			    ImageItem *imgPtr));
 static int		ConfigureImage _ANSI_ARGS_((Tcl_Interp *interp,
@@ -126,10 +100,10 @@ Tk_ItemType tkImageType = {
     ImageCoords,			/* coordProc */
     DeleteImage,			/* deleteProc */
     DisplayImage,			/* displayProc */
-    TK_ITEM_VISITOR_SUPPORT|TK_CONFIG_OBJS,/* flags */
+    0,					/* alwaysRedraw */
     ImageToPoint,			/* pointProc */
     ImageToArea,			/* areaProc */
-    ImageToPostscript,	                /* postscriptProc */
+    (Tk_ItemPostscriptProc *) NULL,	/* postscriptProc */
     ScaleImage,				/* scaleProc */
     TranslateImage,			/* translateProc */
     (Tk_ItemIndexProc *) NULL,		/* indexProc */
@@ -137,11 +111,7 @@ Tk_ItemType tkImageType = {
     (Tk_ItemSelectionProc *) NULL,	/* selectionProc */
     (Tk_ItemInsertProc *) NULL,		/* insertProc */
     (Tk_ItemDCharsProc *) NULL,		/* dTextProc */
-    (Tk_ItemType *) NULL,		/* nextPtr */
-    (Tk_ItemBboxProc *) ComputeImageBbox,/* bboxProc */
-    Tk_Offset(Tk_VisitorType, visitImage), /* acceptProc */
-    (Tk_ItemGetCoordProc *) NULL,	/* getCoordProc */
-    (Tk_ItemSetCoordProc *) NULL	/* setCoordProc */
+    (Tk_ItemType *) NULL		/* nextPtr */
 };
 
 /*
@@ -174,21 +144,8 @@ CreateImage(interp, canvas, itemPtr, argc, argv)
     char **argv;			/* Arguments describing rectangle. */
 {
     ImageItem *imgPtr = (ImageItem *) itemPtr;
-    int i;
 
-    if (argc==1) {
-	i = 1;
-    } else {
-	char *arg = Tcl_GetStringFromObj(args[1], NULL);
-	if (((argc>1) && (arg[0] == '-')
-		&& (arg[1] >= 'a') && (arg[1] <= 'z'))) {
-	    i = 1;
-	} else {
-	    i = 2;
-	}
-    }
-
-    if (argc < i) {
+    if (argc < 2) {
 	Tcl_AppendResult(interp, "wrong # args: should be \"",
 		Tk_PathName(Tk_CanvasTkwin(canvas)), " create ",
 		itemPtr->typePtr->name, " x y ?options?\"",
@@ -203,26 +160,23 @@ CreateImage(interp, canvas, itemPtr, argc, argv)
     imgPtr->canvas = canvas;
     imgPtr->anchor = TK_ANCHOR_CENTER;
     imgPtr->imageString = NULL;
-    imgPtr->activeImageString = NULL;
-    imgPtr->disabledImageString = NULL;
     imgPtr->image = NULL;
-    imgPtr->activeImage = NULL;
-    imgPtr->disabledImage = NULL;
 
     /*
      * Process the arguments to fill in the item record.
      */
 
-    if ((ImageCoords(interp, canvas, itemPtr, i, argv) != TCL_OK)) {
-	goto error;
-    }
-    if (ConfigureImage(interp, canvas, itemPtr, argc-i, argv+i, 0) == TCL_OK) {
-	return TCL_OK;
+    if ((Tk_CanvasGetCoord(interp, canvas, argv[0], &imgPtr->x) != TCL_OK)
+	    || (Tk_CanvasGetCoord(interp, canvas, argv[1], &imgPtr->y)
+		!= TCL_OK)) {
+	return TCL_ERROR;
     }
 
-    error:
-    DeleteImage(canvas, itemPtr, Tk_Display(Tk_CanvasTkwin(canvas)));
-    return TCL_ERROR;
+    if (ConfigureImage(interp, canvas, itemPtr, argc-2, argv+2, 0) != TCL_OK) {
+	DeleteImage(canvas, itemPtr, Tk_Display(Tk_CanvasTkwin(canvas)));
+	return TCL_ERROR;
+    }
+    return TCL_OK;
 }
 
 /*
@@ -255,36 +209,20 @@ ImageCoords(interp, canvas, itemPtr, argc, argv)
 					 * x2, y2, ... */
 {
     ImageItem *imgPtr = (ImageItem *) itemPtr;
-    char x[TCL_DOUBLE_SPACE];
+    char x[TCL_DOUBLE_SPACE], y[TCL_DOUBLE_SPACE];
 
     if (argc == 0) {
-	Tcl_Obj *obj = Tcl_NewObj();
-	Tcl_Obj *subobj = Tcl_NewDoubleObj(imgPtr->x);
-	Tcl_ListObjAppendElement(interp, obj, subobj);
-	subobj = Tcl_NewDoubleObj(imgPtr->y);
-	Tcl_ListObjAppendElement(interp, obj, subobj);
-	Tcl_SetObjResult(interp, obj);
-    } else if (argc < 3) {
-	if (argc==1) {
-	    if (Tcl_ListObjGetElements(interp, args[0], &argc, &args) != TCL_OK) {
-		return TCL_ERROR;
-	    } else if (argc != 2) {
-		sprintf(x,"%d",argc);
-		Tcl_AppendResult(interp, "wrong # coordinates: expected 2, got ",
-		x, (char *) NULL);
-		return TCL_ERROR;
-	    }
-	}
-	if ((Tk_CanvasGetCoordFromObj(interp, canvas, args[0], &imgPtr->x) != TCL_OK)
-		|| (Tk_CanvasGetCoordFromObj(interp, canvas, args[1],
-  		    &imgPtr->y) != TCL_OK)) {
+	Tcl_DoubleResults(interp, 2, 1, imgPtr->x, imgPtr->y); 
+    } else if (argc == 2) {
+	if ((Tk_CanvasGetCoord(interp, canvas, argv[0], &imgPtr->x) != TCL_OK)
+		|| (Tk_CanvasGetCoord(interp, canvas, argv[1],
+		    &imgPtr->y) != TCL_OK)) {
 	    return TCL_ERROR;
 	}
 	ComputeImageBbox(canvas, imgPtr);
     } else {
-	sprintf(x,"%d",argc);
-	Tcl_AppendResult(interp, "wrong # coordinates: expected 0 or 2, got ",
-	x, (char *) NULL);
+	sprintf(interp->result,
+		"wrong # coordinates: expected 0 or 2, got %d", argc);
 	return TCL_ERROR;
     }
     return TCL_OK;
@@ -322,8 +260,8 @@ ConfigureImage(interp, canvas, itemPtr, argc, argv, flags)
     Tk_Image image;
 
     tkwin = Tk_CanvasTkwin(canvas);
-    if (Tk_ConfigureWidget(interp, tkwin, configSpecs, argc, argv,
-	    (char *) imgPtr, flags|TK_CONFIG_OBJS) != TCL_OK) {
+    if (Tk_ConfigureWidget(interp, tkwin, configSpecs, argc,
+	    argv, (char *) imgPtr, flags) != TCL_OK) {
 	return TCL_ERROR;
     }
 
@@ -334,11 +272,6 @@ ConfigureImage(interp, canvas, itemPtr, argc, argv, flags)
      * if it hasn't changed.
      */
 
-    if (imgPtr->activeImageString != NULL) {
-	itemPtr->redraw_flags |= TK_ITEM_STATE_DEPENDANT;
-    } else {
-	itemPtr->redraw_flags &= ~TK_ITEM_STATE_DEPENDANT;
-    }
     if (imgPtr->imageString != NULL) {
 	image = Tk_GetImage(interp, tkwin, imgPtr->imageString,
 		ImageChangedProc, (ClientData) imgPtr);
@@ -352,32 +285,6 @@ ConfigureImage(interp, canvas, itemPtr, argc, argv, flags)
 	Tk_FreeImage(imgPtr->image);
     }
     imgPtr->image = image;
-    if (imgPtr->activeImageString != NULL) {
-	image = Tk_GetImage(interp, tkwin, imgPtr->activeImageString,
-		ImageChangedProc, (ClientData) imgPtr);
-	if (image == NULL) {
-	    return TCL_ERROR;
-	}
-    } else {
-	image = NULL;
-    }
-    if (imgPtr->activeImage != NULL) {
-	Tk_FreeImage(imgPtr->activeImage);
-    }
-    imgPtr->activeImage = image;
-    if (imgPtr->disabledImageString != NULL) {
-	image = Tk_GetImage(interp, tkwin, imgPtr->disabledImageString,
-		ImageChangedProc, (ClientData) imgPtr);
-	if (image == NULL) {
-	    return TCL_ERROR;
-	}
-    } else {
-	image = NULL;
-    }
-    if (imgPtr->disabledImage != NULL) {
-	Tk_FreeImage(imgPtr->disabledImage);
-    }
-    imgPtr->disabledImage = image;
     ComputeImageBbox(canvas, imgPtr);
     return TCL_OK;
 }
@@ -411,21 +318,9 @@ DeleteImage(canvas, itemPtr, display)
     if (imgPtr->imageString != NULL) {
 	ckfree(imgPtr->imageString);
     }
-    if (imgPtr->activeImageString != NULL) {
-	ckfree(imgPtr->activeImageString);
-    }
-    if (imgPtr->disabledImageString != NULL) {
-	ckfree(imgPtr->disabledImageString);
-    }
     if (imgPtr->image != NULL) {
 	Tk_FreeImage(imgPtr->image);
     }
-    if (imgPtr->activeImage != NULL) {
-	Tk_FreeImage(imgPtr->activeImage);
-    }
-    if (imgPtr->disabledImage != NULL) {
-	Tk_FreeImage(imgPtr->disabledImage);
-    }
 }
 
 /*
@@ -457,27 +352,11 @@ ComputeImageBbox(canvas, imgPtr)
 {
     int width, height;
     int x, y;
-    Tk_Image image;
-    Tk_State state = imgPtr->header.state;
-
-    if(state == TK_STATE_NULL) {
-	state = ((TkCanvas *)canvas)->canvas_state;
-    }
-    image = imgPtr->image;
-    if (((TkCanvas *)canvas)->currentItemPtr == (Tk_Item *)imgPtr) {
-	if (imgPtr->activeImage != NULL) {
-	    image = imgPtr->activeImage;
-	}
-    } else if (state == TK_STATE_DISABLED) {
-	if (imgPtr->disabledImage != NULL) {
-	    image = imgPtr->disabledImage;
-	}
-    }
 
     x = (int) (imgPtr->x + ((imgPtr->x >= 0) ? 0.5 : - 0.5));
     y = (int) (imgPtr->y + ((imgPtr->y >= 0) ? 0.5 : - 0.5));
 
-    if ((state == TK_STATE_HIDDEN) || (image == None)) {
+    if (imgPtr->image == None) {
 	imgPtr->header.x1 = imgPtr->header.x2 = x;
 	imgPtr->header.y1 = imgPtr->header.y2 = y;
 	return;
@@ -487,7 +366,7 @@ ComputeImageBbox(canvas, imgPtr)
      * Compute location and size of image, using anchor information.
      */
 
-    Tk_SizeOfImage(image, &width, &height);
+    Tk_SizeOfImage(imgPtr->image, &width, &height);
     switch (imgPtr->anchor) {
 	case TK_ANCHOR_N:
 	    x -= width/2;
@@ -561,25 +440,8 @@ DisplayImage(canvas, itemPtr, display, drawable, x, y, width, height)
 {
     ImageItem *imgPtr = (ImageItem *) itemPtr;
     short drawableX, drawableY;
-    Tk_Image image;
-    Tk_State state = itemPtr->state;
 
-    if(state == TK_STATE_NULL) {
-	state = ((TkCanvas *)canvas)->canvas_state;
-    }
-
-    image = imgPtr->image;
-    if (((TkCanvas *)canvas)->currentItemPtr == itemPtr) {
-	if (imgPtr->activeImage != NULL) {
-	    image = imgPtr->activeImage;
-	}
-    } else if (state == TK_STATE_DISABLED) {
-	if (imgPtr->disabledImage != NULL) {
-	    image = imgPtr->disabledImage;
-	}
-    }
-
-    if (image == NULL) {
+    if (imgPtr->image == NULL) {
 	return;
     }
 
@@ -589,7 +451,7 @@ DisplayImage(canvas, itemPtr, display, drawable, x, y, width, height)
 
     Tk_CanvasDrawableCoords(canvas, (double) x, (double) y,
 	    &drawableX, &drawableY);
-    Tk_RedrawImage(image, x - imgPtr->header.x1, y - imgPtr->header.y1,
+    Tk_RedrawImage(imgPtr->image, x - imgPtr->header.x1, y - imgPtr->header.y1,
 	    width, height, drawable, drawableX, drawableY);
 }
 
@@ -695,100 +557,6 @@ ImageToArea(canvas, itemPtr, rectPtr)
     return 0;
 }
 
-/*
- *--------------------------------------------------------------
- *
- * ImageToPostscript --
- *
- *	This procedure is called to generate Postscript for
- *	image items.
- *
- * Results:
- *	The return value is a standard Tcl result.  If an error
- *	occurs in generating Postscript then an error message is
- *	left in interp->result, replacing whatever used to be there.
- *	If no error occurs, then Postscript for the item is appended
- *	to the result.
- *
- * Side effects:
- *	None.
- *
- *--------------------------------------------------------------
- */
-
-static int
-ImageToPostscript(interp, canvas, itemPtr, prepass)
-    Tcl_Interp *interp;			/* Leave Postscript or error message
-					 * here. */
-    Tk_Canvas canvas;			/* Information about overall canvas. */
-    Tk_Item *itemPtr;			/* Item for which Postscript is
-					 * wanted. */
-    int prepass;			/* 1 means this is a prepass to
-					 * collect font information;  0 means
-					 * final Postscript is being created.*/
-{
-#if defined(__WIN32__) || defined(MAC_TCL)
-    return TCL_OK;
-#else
-    ImageItem *imgPtr = (ImageItem *)itemPtr;
-    Tk_Window canvasWin = Tk_CanvasTkwin(canvas);
-
-    char buffer[256];
-    double x, y;
-    int width, height;
-    Tk_Image image;
-    Tk_State state = itemPtr->state;
-
-    if(state == TK_STATE_NULL) {
-	state = ((TkCanvas *)canvas)->canvas_state;
-    }
-
-    image = imgPtr->image;
-    if (((TkCanvas *)canvas)->currentItemPtr == itemPtr) {
-	if (imgPtr->activeImage != NULL) {
-	    image = imgPtr->activeImage;
-	}
-    } else if (state == TK_STATE_DISABLED) {
-	if (imgPtr->disabledImage != NULL) {
-	    image = imgPtr->disabledImage;
-	}
-    }
-    Tk_SizeOfImage(image, &width, &height);
-
-    /*
-     * Compute the coordinates of the lower-left corner of the image,
-     * taking into account the anchor position for the image.
-     */
-
-    x = imgPtr->x;
-    y = Tk_CanvasPsY(canvas, imgPtr->y);
-    
-    switch (imgPtr->anchor) {
-	case TK_ANCHOR_NW:			y -= height;		break;
-	case TK_ANCHOR_N:	x -= width/2.0; y -= height;		break;
-	case TK_ANCHOR_NE:	x -= width;	y -= height;		break;
-	case TK_ANCHOR_E:	x -= width;	y -= height/2.0;	break;
-	case TK_ANCHOR_SE:	x -= width;				break;
-	case TK_ANCHOR_S:	x -= width/2.0;				break;
-	case TK_ANCHOR_SW:						break;
-	case TK_ANCHOR_W:			y -= height/2.0;	break;
-	case TK_ANCHOR_CENTER:	x -= width/2.0; y -= height/2.0;	break;
-    }
-
-    if (image == NULL) {
-        return TCL_OK;
-    }
-    
-    if (!prepass) {
-	sprintf(buffer, "%.15g %.15g", x, y);
-	Tcl_AppendResult(interp, buffer, " translate\n", (char *) NULL);
-    }
-
-    return Tk_PostscriptImage(image, interp, canvasWin,
-	    ((TkCanvas *) canvas)->psInfo, 0, 0, width, height, prepass);
-#endif
-}
-
 /*
  *--------------------------------------------------------------
  *
