@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkTextTag.c,v 1.2 1998/09/14 18:23:19 stanton Exp $
+ * SCCS: @(#) tkTextTag.c 1.39 97/02/07 13:51:52
  */
 
 #include "default.h"
@@ -23,28 +23,6 @@
  * Information used for parsing tag configuration information:
  */
 
-static Tk_CustomOption stateOption = {
-    Tk_StateParseProc,
-    Tk_StatePrintProc, (ClientData) 2
-};
-
-/* The "-elide" is only provided for compatibility with TkMan 2.0,
- * but might be removed in the future. The option "-state" provides
- * the same functionality and is preferred.
- */
-
-static int	ElideParseProc _ANSI_ARGS_((ClientData clientData,
-		    Tcl_Interp *interp, Tk_Window tkwin,
-		    Arg value, char *widgRec, int offset));
-static Arg	ElidePrintProc _ANSI_ARGS_((ClientData clientData,
-		    Tk_Window tkwin, char *widgRec, int offset,
-		    Tcl_FreeProc **freeProcPtr));
-
-static Tk_CustomOption elideOption = {
-    ElideParseProc,
-    ElidePrintProc, (ClientData) 0
-};
-
 static Tk_ConfigSpec tagConfigSpecs[] = {
     {TK_CONFIG_BORDER, "-background", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(TkTextTag, border), TK_CONFIG_NULL_OK},
@@ -53,9 +31,6 @@ static Tk_ConfigSpec tagConfigSpecs[] = {
     {TK_CONFIG_STRING, "-borderwidth", (char *) NULL, (char *) NULL,
 	"0", Tk_Offset(TkTextTag, bdString),
 		TK_CONFIG_DONT_SET_DEFAULT|TK_CONFIG_NULL_OK},
-    {TK_CONFIG_CUSTOM, "-elide", (char *) NULL, (char *) NULL,
-	(char *) NULL, Tk_Offset(TkTextTag, state),
-	TK_CONFIG_NULL_OK, &elideOption},
     {TK_CONFIG_BITMAP, "-fgstipple", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(TkTextTag, fgStipple), TK_CONFIG_NULL_OK},
     {TK_CONFIG_FONT, "-font", (char *) NULL, (char *) NULL,
@@ -83,17 +58,14 @@ static Tk_ConfigSpec tagConfigSpecs[] = {
 	(char *) NULL, Tk_Offset(TkTextTag, spacing2String), TK_CONFIG_NULL_OK},
     {TK_CONFIG_STRING, "-spacing3", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(TkTextTag, spacing3String), TK_CONFIG_NULL_OK},
-    {TK_CONFIG_CUSTOM, "-state", (char *) NULL, (char *) NULL,
-	(char *) NULL, Tk_Offset(TkTextTag, state),
-	TK_CONFIG_NULL_OK, &stateOption},
     {TK_CONFIG_LANGARG, "-tabs", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(TkTextTag, tabString), TK_CONFIG_NULL_OK},
     {TK_CONFIG_LANGARG, "-underline", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(TkTextTag, underlineString),
 	TK_CONFIG_NULL_OK},
-    {TK_CONFIG_CUSTOM, "-wrap", (char *) NULL, (char *) NULL,
+    {TK_CONFIG_UID, "-wrap", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(TkTextTag, wrapMode),
-	TK_CONFIG_NULL_OK, &textWrapModeOption},
+	TK_CONFIG_NULL_OK},
     {TK_CONFIG_LANGARG, "-data", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(TkTextTag, userData),
 	TK_CONFIG_NULL_OK},
@@ -410,11 +382,14 @@ TkTextTagCmd(textPtr, interp, argc, argv)
 		    return TCL_ERROR;
 		}
 	    }
-	    if (tagPtr->elideString != NULL) {
-		if (Tcl_GetBoolean(interp, tagPtr->elideString,
-			&tagPtr->elide) != TCL_OK) {
-		    return TCL_ERROR;
-		}
+	    if ((tagPtr->wrapMode != NULL)
+		    && (tagPtr->wrapMode != tkTextCharUid)
+		    && (tagPtr->wrapMode != tkTextNoneUid)
+		    && (tagPtr->wrapMode != tkTextWordUid)) {
+		Tcl_AppendResult(interp, "bad wrap mode \"", tagPtr->wrapMode,
+			"\": must be char, none, or word", (char *) NULL);
+		tagPtr->wrapMode = NULL;
+		return TCL_ERROR;
 	    }
 
 	    /*
@@ -448,8 +423,7 @@ TkTextTagCmd(textPtr, interp, argc, argv)
 		    || (tagPtr->spacing3String != NULL)
 		    || (tagPtr->tabString != NULL)
 		    || (tagPtr->underlineString != NULL)
-		    || (tagPtr->state != TK_STATE_NULL)
-		    || (tagPtr->wrapMode != TEXT_WRAPMODE_NULL)) {
+		    || (tagPtr->wrapMode != NULL)) {
 		tagPtr->affectsDisplay = 1;
 	    }
 	    TkTextRedrawTag(textPtr, (TkTextIndex *) NULL,
@@ -842,8 +816,7 @@ TkTextCreateTag(textPtr, tagName)
     tagPtr->tabArrayPtr = NULL;
     tagPtr->underlineString = NULL;
     tagPtr->underline = 0;
-    tagPtr->state = TK_STATE_NULL;
-    tagPtr->wrapMode = TEXT_WRAPMODE_NULL;
+    tagPtr->wrapMode = NULL;
     tagPtr->userData = NULL;
     tagPtr->affectsDisplay = 0;
     textPtr->numTags++;
@@ -937,9 +910,6 @@ TkTextFreeTag(textPtr, tagPtr)
     if (tagPtr->underlineString != NULL) {
 	LangFreeArg(tagPtr->underlineString,TCL_DYNAMIC);
     }
-    if (tagPtr->elideString != NULL) {
-	LangFreeArg(tagPtr->elideString,TCL_DYNAMIC);
-    }
     if (tagPtr->justifyString != NULL) {
 	ckfree(tagPtr->justifyString);
     }
@@ -1415,90 +1385,3 @@ TkTextPickCurrent(textPtr, eventPtr)
 	ckfree((char *) copyArrayPtr);
     }
 }
-
-/*
- *--------------------------------------------------------------
- *
- * ElideParseProc --
- *
- *	This procedure is invoked during option processing to handle
- *	the "-elide" option. This option is an obsolete equivalent
- *	of the "-state" option. It is only added for compatibility
- *	with TkMan 2.0, but will be removed in the future.
- *
- * Results:
- *	A standard Tcl return value.
- *
- * Side effects:
- *	The state for a given item gets replaced by the state
- *	indicated in the value argument.
- *
- *--------------------------------------------------------------
- */
-
-static int
-ElideParseProc(clientData, interp, tkwin, value, widgRec, offset)
-    ClientData clientData;		/* some flags.*/
-    Tcl_Interp *interp;			/* Used for reporting errors. */
-    Tk_Window tkwin;			/* Window containing canvas widget. */
-    Arg value;				/* Value of option. */
-    char *widgRec;			/* Pointer to record for item. */
-    int offset;				/* Offset into item. */
-{
-    int b;
-    register Tk_State *statePtr = (Tk_State *) (widgRec + offset);
-    char *svalue = LangString(value);
-
-    if(svalue == NULL || *svalue == 0) {
-	*statePtr = TK_STATE_NULL;
-	return TCL_OK;
-    }
-    if (Tcl_GetBoolean(interp, value, &b) != TCL_OK) {
-	*statePtr = TK_STATE_NULL;
-	return TCL_ERROR;
-    }
-    *statePtr = b?TK_STATE_HIDDEN:TK_STATE_NORMAL;
-    return TCL_OK;
-}
-/*
- *--------------------------------------------------------------
- *
- * ElidePrintProc --
- *
- *	This procedure is invoked by the Tk configuration code
- *	to produce a printable string for the "-elide"
- *	configuration option.
- *
- * Results:
- *	The return value is a string describing the state for
- *	the item referred to by "widgRec".  In addition, *freeProcPtr
- *	is filled in with the address of a procedure to call to free
- *	the result string when it's no longer needed (or NULL to
- *	indicate that the string doesn't need to be freed).
- *
- * Side effects:
- *	None.
- *
- *--------------------------------------------------------------
- */
-
-static Arg
-ElidePrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
-    ClientData clientData;		/* Ignored. */
-    Tk_Window tkwin;			/* Window containing text widget. */
-    char *widgRec;			/* Pointer to record for item. */
-    int offset;				/* Offset into item. */
-    Tcl_FreeProc **freeProcPtr;		/* Pointer to variable to fill in with
-					 * information about how to reclaim
-					 * storage for return string. */
-{
-    register Tk_State *statePtr = (Tk_State *) (widgRec + offset);
-
-    if (*statePtr==TK_STATE_HIDDEN) {
-	return Tcl_NewIntObj(1);
-    } else if (*statePtr==TK_STATE_NULL) {
-	return LangStringArg("");
-    } else {
-	return Tcl_NewIntObj(0);
-    }
-}
