@@ -3,24 +3,52 @@
  *
  *	This file implements the "Display Item Styles" in the Tix library.
  *
+ * Copyright (c) 1996, Expert Interface Technologies
+ *
+ * See the file "license.terms" for information on usage and redistribution
+ * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
+ *
  *
  */
 
-#include "tkPort.h"
-#include "tkInt.h"
+#include "tixPort.h"
+#include "tix.h"
 #include "tixInt.h"
 
-static Tix_DItemStyle* FindDefaultStyle _ANSI_ARGS_((Tix_DItemInfo * diTypePtr,
-                       Tk_Window tkwin));
+#ifdef _LANG
+#define FORWARD extern
+#define LINKAGE
+#define DItemStyleParseProc TixDItemStyleParseProc
+#define DItemStylePrintProc TixDItemStylePrintProc
+#else
+#define FORWARD static 
+#define LINKAGE static 
+#endif
 
 
-static int   		DItemStyleParseProc _ANSI_ARGS_((ClientData clientData,
+
+typedef struct StyleLink {
+    Tix_DItemInfo * diTypePtr;
+    Tix_DItemStyle* stylePtr;
+    struct StyleLink * next;
+} StyleLink;
+
+typedef struct StyleInfo {
+    Tix_StyleTemplate * tmplPtr;
+    Tix_StyleTemplate tmpl;
+    StyleLink * linkHead;
+} StyleInfo;
+
+
+FORWARD int   		DItemStyleParseProc _ANSI_ARGS_((ClientData clientData,
 			    Tcl_Interp *interp, Tk_Window tkwin,
 			    Arg value,char *widRec, int offset));
-static Arg 		DItemStylePrintProc _ANSI_ARGS_((
+FORWARD Arg 		DItemStylePrintProc _ANSI_ARGS_((
 			    ClientData clientData, Tk_Window tkwin, 
 			    char *widRec, int offset,
 			    Tcl_FreeProc **freeProcPtr));
+static Tix_DItemStyle*	FindDefaultStyle _ANSI_ARGS_((
+			    Tix_DItemInfo * diTypePtr, Tk_Window tkwin));
 static Tix_DItemStyle*	FindStyle _ANSI_ARGS_((
 			    char *styleName));
 static Tix_DItemStyle* 	GetDItemStyle  _ANSI_ARGS_((
@@ -31,8 +59,7 @@ static void		ListAdd _ANSI_ARGS_((Tix_DItemStyle * stylePtr,
 			    Tix_DItem *iPtr));
 static void		ListDelete _ANSI_ARGS_((Tix_DItemStyle * stylePtr,
 			    Tix_DItem *iPtr));
-static void		ListDeleteAll _ANSI_ARGS_((Tix_DItemStyle * stylePtr
-			    ));
+static void		ListDeleteAll _ANSI_ARGS_((Tix_DItemStyle * stylePtr));
 static void		StyleCmdDeletedProc _ANSI_ARGS_((
 			    ClientData clientData));
 static int		StyleCmd _ANSI_ARGS_((ClientData clientData,
@@ -40,17 +67,19 @@ static int		StyleCmd _ANSI_ARGS_((ClientData clientData,
 static int		StyleConfigure _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tix_DItemStyle* stylePtr, int argc,
 			    Arg *args, int flags));
-static void		StyleDestroy _ANSI_ARGS_((
-			    ClientData clientData));	
-static void		DeleteStyle _ANSI_ARGS_((Tix_DItemStyle * stylePtr
-			    ));
+static void		StyleDestroy _ANSI_ARGS_((char *clientData));
+static void		DeleteStyle _ANSI_ARGS_((Tix_DItemStyle * stylePtr));
 static void		DefWindowStructureProc _ANSI_ARGS_((
 			    ClientData clientData, XEvent *eventPtr));
+static void		RefWindowStructureProc _ANSI_ARGS_((
+			    ClientData clientData, XEvent *eventPtr));
+static void		SetDefaultStyle _ANSI_ARGS_((Tix_DItemInfo *diTypePtr,
+			    Tk_Window tkwin, Tix_DItemStyle * stylePtr));
 
 static TIX_DECLARE_SUBCMD(StyleConfigCmd);
 static TIX_DECLARE_SUBCMD(StyleCGetCmd);
 static TIX_DECLARE_SUBCMD(StyleDeleteCmd);
-static TIX_DECLARE_SUBCMD(Tix_ItemStyleCmd);
+extern TIX_DECLARE_SUBCMD(Tix_ItemStyleCmd);
 
 static Tcl_HashTable styleTable;
 static Tcl_HashTable defaultTable;
@@ -99,7 +128,7 @@ void TixDItemStyleFree(iPtr, stylePtr)
  *
  *--------------------------------------------------------------
  */
-static int
+int
 Tix_ItemStyleCmd(clientData, interp, argc, args)
     ClientData clientData;
     Tcl_Interp *interp;		/* Current interpreter. */
@@ -128,7 +157,12 @@ Tix_ItemStyleCmd(clientData, interp, argc, args)
 	return TCL_ERROR;
     }
     
-    /* Parse the -window option */
+    /*
+     * Parse the -refwindow option: this tells the style to use this
+     * window to query the default values for background, foreground
+     * etc. Usually, you should set the -refwindow to the window that
+     * holds the display items which are controlled by this style.
+     */
     if (argc > 2) {
 	size_t len;
 	if (argc %2 != 0) {
@@ -154,7 +188,6 @@ Tix_ItemStyleCmd(clientData, interp, argc, args)
 		continue;
 	    }
 
-	  copy:
 	    if (n!=i) {
 		LangSetString(args+n, LangString(args[i]));
 		LangSetString(args+n+1, LangString(args[i+1]));
@@ -165,14 +198,17 @@ Tix_ItemStyleCmd(clientData, interp, argc, args)
     }
 
     if (styleName == NULL) {
-	/* let's make a unique name */
-	sprintf(buff, "style%d", counter++);
+	/*
+	 * No name is given, we'll make a unique name by default
+	 * (ToDo: check if the name has already been used)
+	 */
+	sprintf(buff, "tixStyle%d", counter++);
 	styleName = buff;
     }
 
-    dispData.interp = interp;
+    dispData.interp  = interp;
     dispData.display = Tk_Display(tkwin);
-    dispData.tkwin = tkwin;
+    dispData.tkwin   = tkwin;
 
     if ((stylePtr = GetDItemStyle(&dispData, diTypePtr,
 	 styleName, NULL)) == NULL) {
@@ -182,9 +218,11 @@ Tix_ItemStyleCmd(clientData, interp, argc, args)
 	DeleteStyle(stylePtr);
 	return TCL_ERROR;
     }
+    Tk_CreateEventHandler(tkwin, StructureNotifyMask,
+	    RefWindowStructureProc, (ClientData)stylePtr);
 
     Tcl_ResetResult(interp);
-    Tcl_AppendResult(interp, styleName, NULL);
+    Tcl_ArgResult(interp, LangObjectArg( interp, styleName));
     return TCL_OK;
 }
 
@@ -210,10 +248,10 @@ StyleCmd(clientData, interp, argc, args)
 	Tix_ArraySize(subCmdInfo), 1, TIX_VAR_ARGS, "?option? arg ?arg ...?",
     };
 
-    Tk_Preserve(clientData);
+    Tcl_Preserve(clientData);
     code = Tix_HandleSubCmds(&cmdInfo, subCmdInfo, clientData,
 	interp, argc, args);
-    Tk_Release(clientData);
+    Tcl_Release(clientData);
 
     return code;
 }
@@ -303,39 +341,47 @@ StyleConfigure(interp, stylePtr, argc, args, flags)
     return TCL_OK;
 }
 
+/*----------------------------------------------------------------------
+ * StyleDestroy --
+ *
+ *	Destroy a display style.
+ *----------------------------------------------------------------------
+ */
 static void
 StyleDestroy(clientData)
-    ClientData clientData;
+char *clientData;
 {
     Tix_DItemStyle* stylePtr= (Tix_DItemStyle*) clientData;
 
     if ((stylePtr->base.flags & TIX_STYLE_DEFAULT)) {
 	/*
-	 * If this is the default style for the display items, we can't
-	 * tell the display items that it has lost its style, otherwise
-	 * the ditem will just attempt to create the default style again,
-	 * and we will go into an infinite loop
+	 * If this is the default style for the display items, we
+	 * can't tell the display items that it has lost its style,
+	 * otherwise the ditem will just attempt to create the default
+	 * style again, and we will go into an infinite loop
 	 */
-	if (stylePtr->base.refCount == 0) {
-	    ckfree((char*)stylePtr->base.name);
-	    stylePtr->base.diTypePtr->styleFreeProc(stylePtr);
+	if (stylePtr->base.refCount != 0) {
+	    /*
+	     * If the refcount is not zero, this style will NOT be
+	     * destroyed.  The real destroy will be triggered if all
+	     * DItems associated with this style is destroyed (in the
+	     * ListDelete() function).
+	     *
+	     * If a widget is destroyed, it is the responsibility of the
+	     * widget writer to delete all DItems associated with this
+	     * widget. We can discover memory leak if the widget is
+	     * destroyed but some default styles associated with it still
+	     * exist
+	     */
+	    return;
 	}
-	/*
-	 * If the refcount is not zero, this style will NOT be destroyed.
-	 * The real destroy will be triggered if all DItems associated with
-	 * this style is destroyed (in the function ListDelete).
-	 *
-	 * If a widget is destroyed, it is the responsibility of the widget
-	 * writer to delete all DItems associated with this widget. We can
-	 * discover memory leak if the widget is destroyed but some default
-	 * styles associated with it still exist
-	 */
     } else {
-	ListDeleteAll(stylePtr);
 	stylePtr->base.refCount = 0;
-	ckfree((char*)stylePtr->base.name);
-	stylePtr->base.diTypePtr->styleFreeProc(stylePtr);
     }
+
+    Tcl_DeleteHashTable(&stylePtr->base.items);
+    ckfree((char*)stylePtr->base.name);
+    stylePtr->base.diTypePtr->styleFreeProc(stylePtr);
 }
 
 static void
@@ -346,7 +392,8 @@ StyleCmdDeletedProc(clientData)
 
     stylePtr->base.styleCmd = NULL;
     if (stylePtr->base.flags & TIX_STYLE_DEFAULT) {
-	/* Don't do anything
+	/*
+	 * Don't do anything
 	 * ToDo: maybe should give a background warning:
 	 */
     } else {
@@ -372,24 +419,29 @@ DeleteStyle(stylePtr)
 	if (hashPtr != NULL) {
 	    Tcl_DeleteHashEntry(hashPtr);
 	}
+	ListDeleteAll(stylePtr);
 
-	Tk_EventuallyFree((ClientData)stylePtr, StyleDestroy);
+	Tcl_EventuallyFree((ClientData)stylePtr, StyleDestroy);
     }
 }
+
+/*
+ *----------------------------------------------------------------------
+ * FindDefaultStyle --
+ *
+ *	Return the default style of the given type of ditem for the
+ *	given tkwin, if such a default style exists.
+ *
+ * Results:
+ *	Pointer to the default style or NULL.
+ *
+ * Side effects:
+ *	None.
+ *----------------------------------------------------------------------
+ */
 
-typedef struct StyleLink {
-    Tix_DItemInfo * diTypePtr;
-    Tix_DItemStyle* stylePtr;
-    struct StyleLink * next;
-} StyleLink;
-
-typedef struct StyleInfo {
-    Tix_StyleTemplate * tmplPtr;
-    Tix_StyleTemplate tmpl;
-    StyleLink * linkHead;
-} StyleInfo;
-
-static Tix_DItemStyle* FindDefaultStyle(diTypePtr, tkwin)
+static Tix_DItemStyle*
+FindDefaultStyle(diTypePtr, tkwin)
     Tix_DItemInfo * diTypePtr;
     Tk_Window tkwin;
 {
@@ -430,7 +482,7 @@ static void SetDefaultStyle(diTypePtr, tkwin, stylePtr)
     newPtr->diTypePtr = diTypePtr;
     newPtr->stylePtr  = stylePtr;
 
-    hashPtr=Tcl_CreateHashEntry(&defaultTable, (char*)tkwin, &isNew);
+    hashPtr = Tcl_CreateHashEntry(&defaultTable, (char*)tkwin, &isNew);
 
     if (!isNew) {
 	infoPtr = (StyleInfo *)Tcl_GetHashValue(hashPtr);
@@ -453,11 +505,27 @@ static void SetDefaultStyle(diTypePtr, tkwin, stylePtr)
     infoPtr->linkHead = newPtr;
 }
 
-Tix_DItemStyle* TixGetDefaultDItemStyle(ddPtr, diTypePtr, iPtr, oldStylePtr)
-    Tix_DispData * ddPtr;
-    Tix_DItemInfo * diTypePtr;
-    Tix_DItem *iPtr;
-    Tix_DItemStyle* oldStylePtr;
+/*
+ *----------------------------------------------------------------------
+ * TixGetDefaultDItemStyle --
+ *
+ *	Gets the default style for an item if the application doesn't
+ *	explicitly give it an style with the -style switch.
+ *
+ * Results:
+ *	The default style.
+ *
+ * Side effects:
+ *	
+ *----------------------------------------------------------------------
+ */
+
+Tix_DItemStyle*
+TixGetDefaultDItemStyle(ddPtr, diTypePtr, iPtr, oldStylePtr)
+    Tix_DispData * ddPtr;		/* Info about the display. */
+    Tix_DItemInfo * diTypePtr;		/* Info about the DItem type. */
+    Tix_DItem *iPtr;			/* Get default style for this DItem. */
+    Tix_DItemStyle* oldStylePtr;	/* ?? */
 {
     Tcl_DString dString;
     Tix_DItemStyle* stylePtr;
@@ -467,7 +535,8 @@ Tix_DItemStyle* TixGetDefaultDItemStyle(ddPtr, diTypePtr, iPtr, oldStylePtr)
 	InitHashTables();
     }
 
-    if ((stylePtr = FindDefaultStyle(diTypePtr, ddPtr->tkwin)) == NULL) {
+    stylePtr = FindDefaultStyle(diTypePtr, ddPtr->tkwin);
+    if (stylePtr == NULL) {
 	/*
 	 * Format default name for this style+window
 	 */
@@ -481,7 +550,7 @@ Tix_DItemStyle* TixGetDefaultDItemStyle(ddPtr, diTypePtr, iPtr, oldStylePtr)
 	/*
 	 * Create the new style
 	 */
-	stylePtr = GetDItemStyle(ddPtr, diTypePtr, dString.string, &isNew);
+	stylePtr = GetDItemStyle(ddPtr, diTypePtr, Tcl_DStringValue(&dString), &isNew);
 	if (isNew) {
 	    diTypePtr->styleConfigureProc(stylePtr, 0, NULL, 0);
 	    stylePtr->base.flags |= TIX_STYLE_DEFAULT;
@@ -536,6 +605,20 @@ void Tix_SetDefaultStyleTemplate(tkwin, tmplPtr)
     }
 }
 
+/*
+ *----------------------------------------------------------------------
+ * GetDItemStyle --
+ *
+ *	Returns an ItemStyle with the given name.
+ *
+ * Results:
+ *	Pointer to the given Tix_DItsmStyle.
+ *
+ * Side effects:
+ *	If the style doesn't already exist, it is allocated.
+ *----------------------------------------------------------------------
+ */
+
 static Tix_DItemStyle*
 GetDItemStyle(ddPtr, diTypePtr, styleName, isNew_ret)
     Tix_DispData * ddPtr;
@@ -567,9 +650,9 @@ GetDItemStyle(ddPtr, diTypePtr, styleName, isNew_ret)
 	stylePtr->base.pad[0] 	 = 0;
 	stylePtr->base.pad[1] 	 = 0;
 	stylePtr->base.anchor 	 = TK_ANCHOR_CENTER;
-	stylePtr->base.items  	 = NULL;
 	stylePtr->base.refCount  = 0;
 	stylePtr->base.flags     = 0;
+	Tcl_InitHashTable(&stylePtr->base.items, TCL_ONE_WORD_KEYS);
 
 	Tcl_SetHashValue(hashPtr, (char*)stylePtr);
     }
@@ -594,84 +677,104 @@ static Tix_DItemStyle* FindStyle(styleName)
 
     return (Tix_DItemStyle *)Tcl_GetHashValue(hashPtr);
 }
-    
+
+/*----------------------------------------------------------------------
+ * TixDItemStyleChanged --
+ *
+ *	Tell each Ditem that are affected by this style that the style
+ *	has changed. The Ditems will respond by updating their
+ *	attributes according to the new values of the style.
+ *----------------------------------------------------------------------
+ */
+
 void TixDItemStyleChanged(diTypePtr, stylePtr)
     Tix_DItemInfo * diTypePtr;
     Tix_DItemStyle * stylePtr;
 {
-    /* Tell all items that are associated with stylePtr that its style has
-     * changed
-     */
-    Tix_DItemLink * linkPtr;
+    Tcl_HashSearch hashSearch;
+    Tcl_HashEntry *hashPtr;
+    Tix_DItem * iPtr;
 
-    for (linkPtr=stylePtr->base.items; linkPtr; linkPtr=linkPtr->next) {
-	if (diTypePtr->styleChangedProc != NULL) {
-	    diTypePtr->styleChangedProc(linkPtr->iPtr);
-	}
+    for (hashPtr = Tcl_FirstHashEntry(&stylePtr->base.items, &hashSearch);
+	 hashPtr;
+	 hashPtr = Tcl_NextHashEntry(&hashSearch)) {
+
+	iPtr = (Tix_DItem *)Tcl_GetHashValue(hashPtr);
+	diTypePtr->styleChangedProc(iPtr);
     }
 }
 
-static void ListAdd(stylePtr, iPtr)
+/*----------------------------------------------------------------------
+ * ListAdd --
+ *
+ *	Add an item to the list of items affected by a style.
+ *----------------------------------------------------------------------
+ */
+
+static void
+ListAdd(stylePtr, iPtr)
     Tix_DItemStyle * stylePtr;
     Tix_DItem *iPtr;
 {
-    Tix_DItemLink * linkPtr = (Tix_DItemLink *)ckalloc(sizeof(Tix_DItemLink));
+    Tcl_HashEntry *hashPtr;
+    int isNew;
 
-    linkPtr->iPtr = iPtr;
-    linkPtr->next = stylePtr->base.items;
-    stylePtr->base.items = linkPtr;
+    hashPtr = Tcl_CreateHashEntry(&stylePtr->base.items, (char*)iPtr, &isNew);
+    if (!isNew) {
+	panic("DItem is already associated with style");
+    } else {
+	Tcl_SetHashValue(hashPtr, (char*)iPtr);
+    }
     ++ stylePtr->base.refCount;
 }
 
-static void ListDelete(stylePtr, iPtr)
+static void
+ListDelete(stylePtr, iPtr)
     Tix_DItemStyle * stylePtr;
     Tix_DItem *iPtr;
 {
-    Tix_DItemLink * linkPtr, *lastPtr;
+    Tcl_HashEntry *hashPtr;
 
-    for (lastPtr=linkPtr=stylePtr->base.items; linkPtr;
-	 lastPtr=linkPtr,linkPtr=linkPtr->next) {
-
-	if (linkPtr->iPtr == iPtr) {
-	    -- stylePtr->base.refCount;
-	    if (linkPtr==stylePtr->base.items) {
-		stylePtr->base.items = linkPtr->next;
-	    } else {
-		lastPtr->next = linkPtr->next;
-	    }
-
-	    ckfree((char *)linkPtr);
-	    break;
-	}
+    hashPtr = Tcl_FindHashEntry(&stylePtr->base.items, (char*)iPtr);
+    if (hashPtr == NULL) {
+	panic("DItem is not associated with style");
     }
-    if ((stylePtr->base.flags & TIX_STYLE_DELETED) &&
-	(stylePtr->base.flags & TIX_STYLE_DEFAULT) &&
-	stylePtr->base.refCount == 0) {
-	Tk_EventuallyFree((ClientData)stylePtr, StyleDestroy);
+    Tcl_DeleteHashEntry(hashPtr);
+    stylePtr->base.refCount--;
+
+    if ((stylePtr->base.refCount == 0) &&
+            (stylePtr->base.flags & TIX_STYLE_DELETED) &&
+	    (stylePtr->base.flags & TIX_STYLE_DEFAULT)) {
+	Tcl_EventuallyFree((ClientData)stylePtr, (Tix_FreeProc *)StyleDestroy);
     }
 }
     
-static void ListDeleteAll(stylePtr)
+static void
+ListDeleteAll(stylePtr)
     Tix_DItemStyle * stylePtr;
 {
-    Tix_DItemLink * linkPtr, *toFree;
+    Tcl_HashSearch hashSearch;
+    Tcl_HashEntry *hashPtr;
+    Tix_DItem * iPtr;
 
-    for (linkPtr=stylePtr->base.items; linkPtr;) {
-	toFree = linkPtr;
-	linkPtr = linkPtr->next;
+    for (hashPtr = Tcl_FirstHashEntry(&stylePtr->base.items, &hashSearch);
+	 hashPtr;
+	 hashPtr = Tcl_NextHashEntry(&hashSearch)) {
 
+	iPtr = (Tix_DItem *)Tcl_GetHashValue(hashPtr);
 	if (stylePtr->base.diTypePtr->lostStyleProc != NULL) {
-	    stylePtr->base.diTypePtr->lostStyleProc(toFree->iPtr);
+	    stylePtr->base.diTypePtr->lostStyleProc(iPtr);
 	}
-	ckfree((char *)toFree);
+	Tcl_DeleteHashEntry(hashPtr);
     }
 }
 
-static void InitHashTables()
+static void
+InitHashTables()
 {
     if (tableInited == 0) {
 	Tcl_InitHashTable(&styleTable, TCL_STRING_KEYS);
-	Tcl_InitHashTable(&defaultTable, sizeof(Tk_Window)/sizeof(int));
+	Tcl_InitHashTable(&defaultTable, TCL_ONE_WORD_KEYS);
 	tableInited = 1;
     }
 }
@@ -692,6 +795,7 @@ static void InitHashTables()
  *
  *--------------------------------------------------------------
  */
+
 static void
 DefWindowStructureProc(clientData, eventPtr)
     ClientData clientData;	/* Pointer to record describing window item. */
@@ -724,6 +828,39 @@ DefWindowStructureProc(clientData, eventPtr)
     Tcl_DeleteHashEntry(hashPtr);
 }
 
+/*
+ *--------------------------------------------------------------
+ *
+ * RefWindowStructureProc --
+ *
+ *	This procedure is invoked when the refwindow of a non-default
+ *	style is deleted.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	The style is deleted.
+ *
+ *--------------------------------------------------------------
+ */
+
+static void
+RefWindowStructureProc(clientData, eventPtr)
+    ClientData clientData;	/* Pointer to record describing window item. */
+    XEvent *eventPtr;		/* Describes what just happened. */
+{
+    Tix_DItemStyle * stylePtr = (Tix_DItemStyle *)clientData;
+
+    if (eventPtr->type == DestroyNotify) {
+	/*
+	 * If some DItems are still associated with this window, they
+	 * will receive a "LostStyle" notification.
+	 */
+	DeleteStyle(stylePtr);
+    } 
+}
+
 /*----------------------------------------------------------------------
  *
  *		 The Tix Customed Config Options
@@ -737,9 +874,11 @@ DefWindowStructureProc(clientData, eventPtr)
  * These are declared in <tix.h>
  */
 
+#ifndef _LANG
 Tk_CustomOption tixConfigItemStyle = {
     DItemStyleParseProc, DItemStylePrintProc, 0,
 };
+#endif
 
 /*----------------------------------------------------------------------
  *  DItemStyleParseProc --
@@ -748,7 +887,7 @@ Tk_CustomOption tixConfigItemStyle = {
  *	inside the widget record.
  *----------------------------------------------------------------------
  */
-static int DItemStyleParseProc(clientData, interp, tkwin, value, widRec,offset)
+LINKAGE int DItemStyleParseProc(clientData, interp, tkwin, value, widRec,offset)
     ClientData clientData;
     Tcl_Interp *interp;
     Tk_Window tkwin;
@@ -761,6 +900,7 @@ static int DItemStyleParseProc(clientData, interp, tkwin, value, widRec,offset)
     Tix_DItemStyle  * oldPtr = *ptr;
     Tix_DItemStyle  * newPtr;
 
+
     if (tableInited  == 0) {
 	InitHashTables();
     }
@@ -770,7 +910,18 @@ static int DItemStyleParseProc(clientData, interp, tkwin, value, widRec,offset)
 	 * User gives a NULL string -- meaning he wants the default
 	 * style
 	 */
-	newPtr = NULL;
+	if (oldPtr && oldPtr->base.flags & TIX_STYLE_DEFAULT) {
+	    /*
+	     * This ditem is already associated with a default style. Let's
+	     * keep it.
+	     */
+	    newPtr = oldPtr;
+	} else {
+	    if (oldPtr) {
+		ListDelete(oldPtr, iPtr);
+	    }
+	    newPtr = NULL;
+	}
     } else {
 	if ((newPtr = FindStyle(LangString(value))) == NULL) {
 	    goto not_found;
@@ -784,23 +935,10 @@ static int DItemStyleParseProc(clientData, interp, tkwin, value, widRec,offset)
 	        newPtr->base.diTypePtr->name, " style", NULL);
 	    return TCL_ERROR;
 	}
-    }
-
-    if (oldPtr != newPtr) {
-	if (oldPtr != NULL) {
-	    /*
-	     * Release the old style
-	     */
-	    if (FindStyle(oldPtr->base.name) == NULL) {
-		/* Old style was deleted: some error */
-		panic("old stylePtr was already deleted!");
+	if (oldPtr != newPtr) {
+	    if (oldPtr != NULL) {
+		ListDelete(oldPtr, iPtr);
 	    }
-	    ListDelete(oldPtr, iPtr);
-	}
-	if (newPtr != NULL) {
-	    /*
-	     * Attach to the new style
-	     */
 	    ListAdd(newPtr, iPtr);
 	}
     }
@@ -808,13 +946,13 @@ static int DItemStyleParseProc(clientData, interp, tkwin, value, widRec,offset)
     *ptr = newPtr;
     return TCL_OK;
 
-  not_found:
+not_found:
     Tcl_AppendResult(interp, "Display style \"", value,
 	"\" not found", NULL);
     return TCL_ERROR;
 }
 
-static Arg 
+LINKAGE Arg 
 DItemStylePrintProc(clientData, tkwin, widRec,offset, freeProcPtr)
     ClientData clientData;
     Tk_Window tkwin;
@@ -823,10 +961,10 @@ DItemStylePrintProc(clientData, tkwin, widRec,offset, freeProcPtr)
     Tcl_FreeProc **freeProcPtr;
 {
     Tix_DItemStyle *stylePtr = *((Tix_DItemStyle**)(widRec+offset));
-
+    Arg result = NULL;
     if (stylePtr != NULL) {
-	return LangObjectArg(stylePtr->base.interp, stylePtr->base.name);
-    } else {
-	return 0;
-    }
+        LangSetArg(&result,LangObjectArg(stylePtr->base.interp, 
+                                         stylePtr->base.name));
+    } 
+    return result;
 }
