@@ -4,21 +4,28 @@
  *	This file implements one of the "Display Items" in the Tix library :
  *	Text display items.
  *
+ * Copyright (c) 1996, Expert Interface Technologies
+ *
+ * See the file "license.terms" for information on usage and redistribution
+ * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
+ *
  */
-#include "tkPort.h"
-#include "tkInt.h"
+
+#include "tixPort.h"
 #include "tixInt.h"
-#include "tixDefault.h"
+#include "tkInt.h"
+#include "tixDef.h"
 
 /*----------------------------------------------------------------------
  *
- * 		Configuration options for Text Items
+ *		Configuration options for Text Items
  *
  *----------------------------------------------------------------------
  */
+
 #define DEF_TEXTITEM_STYLE	 ""
 #define DEF_TEXTITEM_TEXT	 ""
-#define DEF_TEXTITEM_UNDERLINE   "-1"
+#define DEF_TEXTITEM_UNDERLINE	 "-1"
 #define DEF_TEXTITEM_TYPE	 "text"
 
 static Tk_ConfigSpec textItemConfigSpecs[] = {
@@ -28,7 +35,7 @@ static Tk_ConfigSpec textItemConfigSpecs[] = {
     {TK_CONFIG_CUSTOM, "-style", "textStyle", "TextStyle",
        DEF_TEXTITEM_STYLE, Tk_Offset(TixTextItem, stylePtr),
        TK_CONFIG_NULL_OK, &tixConfigItemStyle},
-    {TK_CONFIG_STRING, "-text", "text", "Text",
+    {TK_CONFIG_LANGARG, "-text", "text", "Text",
        DEF_TEXTITEM_TEXT, Tk_Offset(TixTextItem, text),
        TK_CONFIG_NULL_OK},
     {TK_CONFIG_INT, "-underline", "underline", "Underline",
@@ -39,7 +46,7 @@ static Tk_ConfigSpec textItemConfigSpecs[] = {
 
 /*----------------------------------------------------------------------
  *
- * 		Configuration options for Text Styles
+ *		Configuration options for Text Styles
  *
  *----------------------------------------------------------------------
  */
@@ -67,12 +74,12 @@ static Tk_ConfigSpec textItemConfigSpecs[] = {
 #define DEF_TEXTSTYLE_DISABLED_BG_COLOR		DISABLED_BG
 #define DEF_TEXTSTYLE_DISABLED_BG_MONO		WHITE
 
-#define DEF_TEXTSTYLE_PADX	 		"2"
-#define DEF_TEXTSTYLE_PADY	 		"2"
-#define DEF_TEXTSTYLE_FONT           "-Adobe-Helvetica-Bold-R-Normal--*-120-*"
-#define DEF_TEXTSTYLE_JUSTIFY	 		"left"
-#define DEF_TEXTSTYLE_WLENGTH	 		"0"
-#define DEF_TEXTSTYLE_ANCHOR 			"w"
+#define DEF_TEXTSTYLE_PADX			"2"
+#define DEF_TEXTSTYLE_PADY			"2"
+#define DEF_TEXTSTYLE_FONT	     "-Adobe-Helvetica-Bold-R-Normal--*-120-*"
+#define DEF_TEXTSTYLE_JUSTIFY			"left"
+#define DEF_TEXTSTYLE_WLENGTH			"0"
+#define DEF_TEXTSTYLE_ANCHOR			"w"
 
 #if 0
     /* %bordercolor not used */
@@ -102,7 +109,7 @@ static Tk_ConfigSpec textStyleConfigSpecs[] = {
                 NULL, 0, 0},
 
     {TK_CONFIG_FONT, "-font", "font", "Font",
-       DEF_TEXTSTYLE_FONT, Tk_Offset(TixTextStyle, fontPtr), 0},
+       DEF_TEXTSTYLE_FONT, Tk_Offset(TixTextStyle, font), 0},
 
     {TK_CONFIG_JUSTIFY, "-justify", "justify", "Justyfy",
        DEF_TEXTSTYLE_JUSTIFY, Tk_Offset(TixTextStyle, justify),
@@ -194,10 +201,12 @@ static Tk_ConfigSpec textStyleConfigSpecs[] = {
  */
 static void		Tix_TextItemCalculateSize  _ANSI_ARGS_((
 			    Tix_DItem * iPtr));
-static int 		Tix_TextItemConfigure _ANSI_ARGS_((
+static char *		Tix_TextItemComponent  _ANSI_ARGS_((
+			    Tix_DItem * iPtr, int x, int y));
+static int		Tix_TextItemConfigure _ANSI_ARGS_((
 			    Tix_DItem * iPtr, int argc, Arg *args,
 			    int flags));
-static Tix_DItem * 	Tix_TextItemCreate _ANSI_ARGS_((
+static Tix_DItem *	Tix_TextItemCreate _ANSI_ARGS_((
 			    Tix_DispData * ddPtr, Tix_DItemInfo * diTypePtr));
 static void		Tix_TextItemDisplay  _ANSI_ARGS_((
 			    Pixmap pixmap, GC gc, Tix_DItem * iPtr,
@@ -206,9 +215,9 @@ static void		Tix_TextItemFree  _ANSI_ARGS_((
 			    Tix_DItem * iPtr));
 static void		Tix_TextItemLostStyle  _ANSI_ARGS_((
 			    Tix_DItem * iPtr));
-static void 		Tix_TextItemStyleChanged  _ANSI_ARGS_((
+static void		Tix_TextItemStyleChanged  _ANSI_ARGS_((
 			    Tix_DItem * iPtr));
-static int 		Tix_TextStyleConfigure _ANSI_ARGS_((
+static int		Tix_TextStyleConfigure _ANSI_ARGS_((
 			    Tix_DItemStyle* style, int argc, Arg *args,
 			    int flags));
 static Tix_DItemStyle *	Tix_TextStyleCreate _ANSI_ARGS_((
@@ -216,7 +225,7 @@ static Tix_DItemStyle *	Tix_TextStyleCreate _ANSI_ARGS_((
 			    Tix_DItemInfo * diTypePtr, char * name));
 static void		Tix_TextStyleFree _ANSI_ARGS_((
 			    Tix_DItemStyle* style));
-static void 		Tix_TextStyleSetTemplate _ANSI_ARGS_((
+static void		Tix_TextStyleSetTemplate _ANSI_ARGS_((
 			    Tix_DItemStyle* style,
 			    Tix_StyleTemplate * tmplPtr));
 
@@ -226,6 +235,7 @@ Tix_DItemInfo tix_TextItemType = {
     Tix_TextItemCreate,		/* createProc */
     Tix_TextItemConfigure,
     Tix_TextItemCalculateSize,
+    Tix_TextItemComponent,
     Tix_TextItemDisplay,
     Tix_TextItemFree,
     Tix_TextItemStyleChanged,
@@ -256,7 +266,7 @@ static Tix_DItem * Tix_TextItemCreate(ddPtr, diTypePtr)
     itPtr = (TixTextItem*) ckalloc(sizeof(TixTextItem));
 
     itPtr->diTypePtr	= &tix_TextItemType;
-    itPtr->ddPtr 	= ddPtr;
+    itPtr->ddPtr	= ddPtr;
 #if 1
     itPtr->stylePtr	= (TixTextStyle*)TixGetDefaultDItemStyle(
 	    itPtr->ddPtr, &tix_TextItemType, (Tix_DItem*)itPtr, NULL);
@@ -271,7 +281,7 @@ static Tix_DItem * Tix_TextItemCreate(ddPtr, diTypePtr)
     itPtr->text		= NULL;
     itPtr->textW	= 0;
     itPtr->textH	= 0;
-    itPtr->underline 	= -1;
+    itPtr->underline	= -1;
 
     return (Tix_DItem *)itPtr;
 }
@@ -320,54 +330,6 @@ static int Tix_TextItemConfigure(iPtr, argc, args, flags)
     return TCL_OK;
 }
 
-#if 0
-    /* %bordercolor not used */
-static void DrawBorder(pixmap, gc, iPtr, x, y, width, height, flags)
-    Pixmap pixmap;
-    GC gc;
-    Tix_DItem * iPtr;
-    int x;
-    int y;
-    int width;
-    int height;
-    int flags;
-{
-    int x1, y1, x2, y2, i;
-
-    TixTextItem *itPtr = (TixTextItem *)iPtr;
-    if (itPtr->stylePtr->borderWidth <= 0) {
-	return;
-    }
-    if (itPtr->stylePtr->relief == TIX_RELIEF_SOLID) {
-	if (!(flags & TIX_DITEM_BOTTOM)) {
-	    x1 = x;
-	    x2 = x + width-1;
-	    y1 = y2 = y + height - 1;
-	    
-	    for (i=0; i<itPtr->stylePtr->borderWidth; i++) {
-		XDrawLine(itPtr->ddPtr->display, pixmap,
-		    itPtr->stylePtr->borderGC, x1, y1, x2, y2);
-		--y1;
-		--y2;
-	    }
-	}
-	if (!(flags & TIX_DITEM_RIGHT)) {
-	    y1 = y;
-	    y2 = y + height -1;
-	    x1 = x2 = x + width - 1;
-	    
-	    for (i=0; i<itPtr->stylePtr->borderWidth; i++) {
-		XDrawLine(itPtr->ddPtr->display, pixmap,
-		    itPtr->stylePtr->borderGC, x1, y1, x2, y2);
-		--x1;
-		--x2;
-	    }
-	}
-
-    }
-}
-#endif
-
 static void Tix_TextItemDisplay(pixmap, gc, iPtr, x, y, width, height, flags)
     Pixmap pixmap;
     GC gc;
@@ -380,37 +342,54 @@ static void Tix_TextItemDisplay(pixmap, gc, iPtr, x, y, width, height, flags)
 {
     TixTextItem *itPtr = (TixTextItem *)iPtr;
     GC foreGC, backGC;
+    TixpSubRegion subReg;
+
+    if ((width <= 0) || (height <= 0)) {
+	return;
+    }
 
     TixGetColorDItemGC(iPtr, &backGC, &foreGC, flags);
 
+    TixpStartSubRegionDraw(itPtr->ddPtr->display, pixmap, foreGC,
+	    &subReg, 0, 0, x, y, width, height,
+	    itPtr->size[0], itPtr->size[1]);
+
     if (backGC != None) {
-	/* Draw the background */
-	XFillRectangle(itPtr->ddPtr->display, pixmap,
-	    backGC,
-	    x, y, width, height);
+	TixpSubRegFillRectangle(itPtr->ddPtr->display, pixmap, backGC,
+		&subReg, x, y, width, height);
     }
-#if 0
-    /* %bordercolor not used */
-    DrawBorder(pixmap, gc, iPtr, x, y, width, height, flags);
-#endif
 
     TixDItemGetAnchor(itPtr->stylePtr->anchor, x, y, width, height,
-	itPtr->size[0], itPtr->size[1], &x, &y);
+	    itPtr->size[0], itPtr->size[1], &x, &y);
 
     if (foreGC != None && itPtr->text != NULL) {
 	x += itPtr->stylePtr->pad[0];
 	y += itPtr->stylePtr->pad[1];
-	
-	TkDisplayText(itPtr->ddPtr->display, pixmap, itPtr->stylePtr->fontPtr,
-       	    itPtr->text, itPtr->numChars,
-	    x, y,
-	    itPtr->textW,
-	    itPtr->stylePtr->justify,
-	    itPtr->underline,
-	    foreGC);
+
+	TixpSubRegDisplayText(itPtr->ddPtr->display, pixmap, foreGC,
+		&subReg, itPtr->stylePtr->font, LangString(itPtr->text),
+		itPtr->numChars, x, y, itPtr->textW, itPtr->stylePtr->justify,
+		itPtr->underline);
     }
+
+    TixpEndSubRegionDraw(itPtr->ddPtr->display, pixmap, foreGC,
+	    &subReg);
 }
 
+static char * Tix_TextItemComponent(iPtr, x, y)
+    Tix_DItem * iPtr;
+    int x;
+    int y;
+{
+#if 0
+    TixTextItem *itPtr = (TixTextItem *)iPtr;
+#endif
+    static char * body = "body";
+
+    return body;
+}
+
+
 static void Tix_TextItemCalculateSize(iPtr)
     Tix_DItem * iPtr;
 {
@@ -418,8 +397,8 @@ static void Tix_TextItemCalculateSize(iPtr)
 
 
     if (itPtr->text) {
-	itPtr->numChars = strlen(itPtr->text);
-	TkComputeTextGeometry(itPtr->stylePtr->fontPtr, itPtr->text,
+	itPtr->numChars = strlen(LangString(itPtr->text));
+	TixComputeTextGeometry(itPtr->stylePtr->font, LangString(itPtr->text),
 		itPtr->numChars,
 		itPtr->stylePtr->wrapLength, &itPtr->textW, &itPtr->textH);
 
@@ -475,7 +454,7 @@ Tix_TextItemLostStyle(iPtr)
 /*----------------------------------------------------------------------
  *
  *
- *  			Display styles
+ *			Display styles
  *
  *
  *----------------------------------------------------------------------
@@ -490,9 +469,12 @@ Tix_TextStyleCreate(interp, tkwin, diTypePtr, name)
     TixTextStyle * stylePtr = (TixTextStyle *)ckalloc(sizeof(TixTextStyle));
     int i;
 
-    stylePtr->fontPtr 	 = NULL;
-    stylePtr->justify 	 = TK_JUSTIFY_LEFT;
+    stylePtr->font	 = NULL;
+    stylePtr->justify	 = TK_JUSTIFY_LEFT;
     stylePtr->wrapLength = 0;
+    stylePtr->pad[0]	 = 0;
+    stylePtr->pad[1]	 = 0;
+    stylePtr->anchor	 = TK_ANCHOR_CENTER;
 
     for (i=0; i<4; i++) {
 	stylePtr->colors[i].bg = NULL;
@@ -525,7 +507,7 @@ Tix_TextStyleConfigure(style, argc, args, flags)
     GC newGC;
     int i, isNew;
 
-    if (stylePtr->fontPtr == NULL) {
+    if (stylePtr->font == NULL) {
 	isNew = 1;
     } else {
 	isNew = 0;
@@ -539,7 +521,7 @@ Tix_TextStyleConfigure(style, argc, args, flags)
 	}
     }
 
-    gcValues.font = stylePtr->fontPtr->fid;
+    gcValues.font = TixFontId(stylePtr->font);
     gcValues.graphics_exposures = False;
 
     for (i=0; i<4; i++) {
@@ -630,12 +612,12 @@ Tix_TextStyleSetTemplate(style, tmplPtr)
     int i;
 
     if (tmplPtr->flags & TIX_DITEM_FONT) {
-	if (stylePtr->fontPtr != NULL) {
-	    Tk_FreeFontStruct(stylePtr->fontPtr);
+	if (stylePtr->font != NULL) {
+	    TixFreeFont(stylePtr->font);
 	}
-	stylePtr->fontPtr = Tk_GetFontStruct(
-  	    stylePtr->interp, stylePtr->tkwin,
-	    Tk_NameOfFontStruct(tmplPtr->fontPtr));
+	stylePtr->font = TixGetFont(
+	    stylePtr->interp, stylePtr->tkwin,
+	    TixNameOfFont(tmplPtr->font));
     }
     if (tmplPtr->flags & TIX_DITEM_PADX) {
 	stylePtr->pad[0] = tmplPtr->pad[0];
@@ -650,7 +632,7 @@ Tix_TextStyleSetTemplate(style, tmplPtr)
 		Tk_FreeColor(stylePtr->colors[i].bg);
 	    }
 	    stylePtr->colors[i].bg = Tk_GetColor(
-  	    	stylePtr->interp, stylePtr->tkwin,
+		stylePtr->interp, stylePtr->tkwin,
 		Tk_NameOfColor(tmplPtr->colors[i].bg));
 	}
     }
@@ -660,7 +642,7 @@ Tix_TextStyleSetTemplate(style, tmplPtr)
 		Tk_FreeColor(stylePtr->colors[i].fg);
 	    }
 	    stylePtr->colors[i].fg = Tk_GetColor(
-  	    	stylePtr->interp, stylePtr->tkwin,
+		stylePtr->interp, stylePtr->tkwin,
 		Tk_NameOfColor(tmplPtr->colors[i].fg));
 	}
     }
@@ -671,7 +653,7 @@ Tix_TextStyleSetTemplate(style, tmplPtr)
 	    Tk_FreeColor(stylePtr->borderColor);
 	}
 	stylePtr->borderColor = Tk_GetColor(
-  	   stylePtr->interp, stylePtr->tkwin,
+	   stylePtr->interp, stylePtr->tkwin,
 	   Tk_NameOfColor(tmplPtr->borderColor));
     }
     if (tmplPtr->flags & TIX_DITEM_BORDER_WIDTH) {
