@@ -3,16 +3,24 @@
  *
  *	This module implements "tixNoteBookFrame" widgets.
  *
+ * Copyright (c) 1996, Expert Interface Technologies
+ *
+ * See the file "license.terms" for information on usage and redistribution
+ * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
+ *
+ *
  */
 
-#include "tkPort.h"
 #include "tkInt.h"
-#include "tix.h"
-#include <tixDefault.h>
+#include "tixPort.h"
+#include "tixInt.h"
+#include "tkVMacro.h"
+#include "tixDef.h"
 
 #define NUM_TAB_POINTS 6
 
-/* A data structure of the following type is kept for each
+/*
+ * A data structure of the following type is kept for each
  * widget managed by this file:
  */
 
@@ -52,7 +60,7 @@ typedef struct NoteBookFrameStruct {
     int isSlave;		/* if is in Slave mode, do not request for
 				 * germetry */
     /* Text drawing */
-    XFontStruct *fontPtr;	/* Information about text font, or NULL. */
+    TixFont font;		/* Information about text font, or NULL. */
     XColor *textColorPtr;	/* Color for drawing text. */
     XColor *disabledFg;		/* Foreground color when disabled.  NULL
 				 * means use normalFg with a 50% stipple
@@ -85,8 +93,6 @@ typedef struct NoteBookFrameStruct {
     unsigned int redrawing : 1;
     unsigned int gotFocus : 1;
 
-    LangCallback *command;
-
 } NoteBookFrame;
 
 typedef struct _Tab {
@@ -111,9 +117,6 @@ typedef struct _Tab {
     char * imageString;
 
     Pixmap bitmap;
-
-    Arg data;			/* user data field */
-
 } Tab;
 
 typedef NoteBookFrame   WidgetRecord;
@@ -152,9 +155,6 @@ static Tk_ConfigSpec configSpecs[] = {
        DEF_NOTEBOOKFRAME_BORDER_WIDTH, Tk_Offset(WidgetRecord, borderWidth),
        0},
 
-    {TK_CONFIG_CALLBACK, "-command", "command", "Command",
-       "", Tk_Offset(WidgetRecord, command), TK_CONFIG_NULL_OK},
-
     {TK_CONFIG_ACTIVE_CURSOR, "-cursor", "cursor", "Cursor",
        DEF_NOTEBOOKFRAME_CURSOR, Tk_Offset(WidgetRecord, cursor),
        TK_CONFIG_NULL_OK},
@@ -181,7 +181,7 @@ static Tk_ConfigSpec configSpecs[] = {
        TK_CONFIG_MONO_ONLY},
 
     {TK_CONFIG_FONT, "-font", "font", "Font",
-       DEF_NOTEBOOKFRAME_FONT, Tk_Offset(WidgetRecord, fontPtr), 0},
+       DEF_NOTEBOOKFRAME_FONT, Tk_Offset(WidgetRecord, font), 0},
 
     {TK_CONFIG_COLOR, "-foreground", "foreground", "Foreground",
        DEF_NOTEBOOKFRAME_FG_COLOR, Tk_Offset(WidgetRecord, textColorPtr),
@@ -203,6 +203,9 @@ static Tk_ConfigSpec configSpecs[] = {
        Tk_Offset(WidgetRecord, inActiveBorder),
        TK_CONFIG_MONO_ONLY},
 
+    {TK_CONFIG_RELIEF, "-relief", "relief", "Relief",
+       DEF_NOTEBOOKFRAME_RELIEF, Tk_Offset(WidgetRecord, relief), 0},
+
     {TK_CONFIG_BOOLEAN, "-slave", "slave", "Slave",
        DEF_NOTEBOOKFRAME_SLAVE, Tk_Offset(WidgetRecord, isSlave), 0},
 
@@ -216,9 +219,6 @@ static Tk_ConfigSpec configSpecs[] = {
 	DEF_NOTEBOOKFRAME_TAKE_FOCUS, Tk_Offset(WidgetRecord, takeFocus),
 	TK_CONFIG_NULL_OK},
 
-    {TK_CONFIG_RELIEF, "-relief", "relief", "Relief",
-       DEF_NOTEBOOKFRAME_RELIEF, Tk_Offset(WidgetRecord, relief), 0},
-
     {TK_CONFIG_PIXELS, "-width", "width", "Width",
        DEF_NOTEBOOKFRAME_WIDTH, Tk_Offset(WidgetRecord, desiredWidth), 0},
     
@@ -242,9 +242,6 @@ static Tk_ConfigSpec tabConfigSpecs[] = {
     {TK_CONFIG_BITMAP, "-bitmap",          NULL,          NULL,
        DEF_NBF_TAB_BITMAP, Tk_Offset(Tab, bitmap), TK_CONFIG_NULL_OK},
 
-    {TK_CONFIG_LANGARG, "-data",          NULL,          NULL,
-       "", Tk_Offset(Tab, data), TK_CONFIG_NULL_OK},
-
     {TK_CONFIG_STRING, "-image",          NULL,          NULL,
        DEF_NBF_TAB_IMAGE, Tk_Offset(Tab, imageString),
        TK_CONFIG_NULL_OK},
@@ -283,7 +280,7 @@ static int		WidgetConfigure _ANSI_ARGS_((Tcl_Interp *interp,
 static int		WidgetCommand _ANSI_ARGS_((ClientData clientData,
 			    Tcl_Interp *, int argc, Arg *args));
 static void		WidgetComputeGeometry _ANSI_ARGS_((WidgetPtr wPtr));
-static void		WidgetDestroy _ANSI_ARGS_((ClientData clientData));
+static void		WidgetDestroy _ANSI_ARGS_((char *clientData));
 static void		WidgetDisplay _ANSI_ARGS_((ClientData clientData));
 static void		WidgetEventProc _ANSI_ARGS_((ClientData clientData,
 			    XEvent *eventPtr));
@@ -295,8 +292,16 @@ static int		AddTab _ANSI_ARGS_((WidgetPtr wPtr, char * name,
 static void		DeleteTab _ANSI_ARGS_((Tab * tPtr));
 static void		CancelRedrawWhenIdle _ANSI_ARGS_((WidgetPtr wPtr));
 static void		ComputeGeometry _ANSI_ARGS_((WidgetPtr wPtr));
+static void		DrawTab _ANSI_ARGS_((WidgetPtr wPtr,
+			    Tab * tPtr, int x, int isActive,
+			    Drawable drawable));
 static Tab * 		FindTab _ANSI_ARGS_((Tcl_Interp *interp,
 			    WidgetPtr wPtr, char * name));
+static void		FocusTab _ANSI_ARGS_((WidgetPtr wPtr,
+			    Tab * tPtr, int x, Drawable drawable));
+static void		GetTabPoints _ANSI_ARGS_((
+			    WidgetPtr wPtr, Tab * tPtr,
+			    int x, XPoint *points));
 static void		ImageProc _ANSI_ARGS_((ClientData clientData,
 			    int x, int y, int width, int height,
 			    int imgWidth, int imgHeight));
@@ -329,7 +334,7 @@ Tix_NoteBookFrameCmd(clientData, interp, argc, args)
     int argc;			/* Number of arguments. */
     Arg *args;		/* Argument strings. */
 {
-    Tk_Window main = (Tk_Window) clientData;
+    Tk_Window mainw = (Tk_Window) clientData;
     WidgetPtr wPtr;
     Tk_Window tkwin;
 
@@ -339,7 +344,7 @@ Tix_NoteBookFrameCmd(clientData, interp, argc, args)
 	return TCL_ERROR;
     }
 
-    tkwin = Tk_CreateWindowFromPath(interp, main, LangString(args[1]),          NULL);
+    tkwin = Tk_CreateWindowFromPath(interp, mainw, LangString(args[1]),          NULL);
     if (tkwin == NULL) {
 	return TCL_ERROR;
     }
@@ -366,7 +371,7 @@ Tix_NoteBookFrameCmd(clientData, interp, argc, args)
     wPtr->disabledGC 		= None;
     wPtr->inActiveBorder 	= NULL;
     wPtr->focusBorder	 	= NULL;
-    wPtr->fontPtr	 	= NULL;
+    wPtr->font	 	= NULL;
     wPtr->textColorPtr	 	= NULL;
     wPtr->textGC	 	= None;
     wPtr->focusGC	 	= None;
@@ -432,7 +437,7 @@ WidgetCommand(clientData, interp, argc, args)
 		LangString(args[0]), " option ?arg arg ...?\"",          NULL);
 	return TCL_ERROR;
     }
-    Tk_Preserve((ClientData) wPtr);
+    Tcl_Preserve((ClientData) wPtr);
     c = LangString(args[1])[0];
     length = strlen(LangString(args[1]));
 
@@ -441,7 +446,7 @@ WidgetCommand(clientData, interp, argc, args)
 	if (argc != 3) {
 	    Tcl_AppendResult(interp, "wrong # args: should be \"",
 		LangString(args[0]), " ", LangString(args[1]), " name\"",          NULL);
-	    result = TCL_ERROR;
+	    goto error;
 	}
 	else {
 	    if (strcmp(LangString(args[2]), "")==0) {
@@ -453,7 +458,6 @@ WidgetCommand(clientData, interp, argc, args)
 		    wPtr->focus  = 0;
 		}
 		RedrawWhenIdle(wPtr);
-		result = TCL_OK;
 	    }
 	    else {
 		Tab * tPtr;
@@ -468,14 +472,13 @@ WidgetCommand(clientData, interp, argc, args)
 			    wPtr->focus  = tPtr;
 			}
 			RedrawWhenIdle(wPtr);
-			result = TCL_OK;
 			goto done;
 		    }
 		}
 
 		Tcl_AppendResult(interp, "unknown tab \"",
 		    LangString(args[0]), "\"",          NULL);
-		result = TCL_ERROR;
+		goto error;
 	    }
 	}
     }
@@ -483,14 +486,14 @@ WidgetCommand(clientData, interp, argc, args)
 	if (argc < 3) {
 	    Tcl_AppendResult(interp, "wrong # args: should be ",
 		LangString(args[0]), " add name ?options?",          NULL);
-	    result = TCL_ERROR;
+	    goto error;
 	}
 	else {
 	    if (AddTab(wPtr, LangString(args[2]), args+3, argc-3)!= TCL_OK) {
-		result = TCL_ERROR;
+		goto error;
 	    } else {
 		WidgetComputeGeometry(wPtr);
-		result = TCL_OK;
+		RedrawWhenIdle(wPtr);
 	    }
 	}
     }
@@ -518,7 +521,7 @@ WidgetCommand(clientData, interp, argc, args)
 	Tab * tPtr, * prev ;
 
 	if (argc != 3) {
-	    result = Tix_ArgcError(interp, argc, args, 2, "page");
+	    Tix_ArgcError(interp, argc, args, 2, "page");
 	    goto error;
 	}
 
@@ -549,7 +552,6 @@ WidgetCommand(clientData, interp, argc, args)
 	DeleteTab(tPtr);
 	ComputeGeometry(wPtr);
 	RedrawWhenIdle(wPtr);
-	result = TCL_OK;
     }
     else if ((c == 'g') && (strncmp(LangString(args[1]), "geometryinfo", length) == 0)) {
 	char buff[20];
@@ -558,7 +560,6 @@ WidgetCommand(clientData, interp, argc, args)
 	sprintf(buff, "%d %d", wPtr->width, wPtr->height);
 
 	Tcl_AppendResult(interp, buff, NULL);
-	result = TCL_OK;
     }
     else if ((c == 'i') && (strncmp(LangString(args[1]), "identify", length) == 0)) {
 	if (argc != 4) {
@@ -583,20 +584,18 @@ WidgetCommand(clientData, interp, argc, args)
 		    right = left + (wPtr->borderWidth +  wPtr->tabPadx) * 2
 		      + tPtr->width;
 
-		    if (x >= left && x <= right && tPtr->state == tkNormalUid){
+		    if (x >= left && x <= right && tPtr->state ==tixNormalUid){
 			Tcl_AppendResult(interp, tPtr->name, NULL);
-			result = TCL_OK;
 			goto done;
 		    }
-
 		    left = right;
 		}
 	    }
 
+	    /*
+	     * An empty string is returned to indicate "nothing selected"
+	     */
 	    Tcl_ResetResult(interp);
-	    result = TCL_OK;	/* an empty string is returned to indicate
-				 * "nothing selected" */
-	    goto done;
 	}
     }
     else if ((c == 'i') && (strncmp(LangString(args[1]), "info", length) == 0)) {
@@ -608,22 +607,16 @@ WidgetCommand(clientData, interp, argc, args)
 	    for (tPtr=wPtr->tabHead; tPtr; tPtr=tPtr->next) {
 		Tcl_AppendElement(interp, tPtr->name);
 	    }
-	    result = TCL_OK;
-	    goto done;
 	}
 	else if (argc == 3 && strcmp(LangString(args[2]), "active")==0 ) {
 	    if (wPtr->active) {
 		Tcl_AppendResult(interp, wPtr->active->name, NULL);
 	    }
-	    result = TCL_OK;
-	    goto done;
 	}
 	else if (argc == 3 && strcmp(LangString(args[2]), "focus")==0 ) {
 	    if (wPtr->focus) {
 		Tcl_AppendResult(interp, wPtr->focus->name, NULL);
 	    }
-	    result = TCL_OK;
-	    goto done;
 	}
 	else if (argc == 3 && strcmp(LangString(args[2]), "focusnext")==0 ) {
 	    Tab * next;
@@ -635,8 +628,6 @@ WidgetCommand(clientData, interp, argc, args)
 		}
 		Tcl_AppendResult(interp, next->name, NULL);
 	    }
-	    result = TCL_OK;
-	    goto done;
 	}
 	else if (argc == 3 && strcmp(LangString(args[2]), "focusprev")==0 ) {
 	    Tab * prev, *tPtr;
@@ -655,8 +646,6 @@ WidgetCommand(clientData, interp, argc, args)
 	    if (prev) {
 		Tcl_AppendResult(interp, prev->name, NULL);
 	    }
-	    result = TCL_OK;
-	    goto done;
 	}
 	else {
 	    Tcl_AppendResult(interp, "wrong number of arguments or ",
@@ -671,7 +660,7 @@ WidgetCommand(clientData, interp, argc, args)
 	Tab * tPtr;
 
 	if (argc != 4) {
-	    result =  Tix_ArgcError(interp, argc, args, 2, "option");
+	    Tix_ArgcError(interp, argc, args, 2, "option");
 	    goto error;
 	}
 	if ((tPtr=FindTab(interp, wPtr, LangString(args[2]))) == NULL) {
@@ -684,7 +673,7 @@ WidgetCommand(clientData, interp, argc, args)
 	Tab * tPtr;
 
 	if (argc < 3) {
-	    result =  Tix_ArgcError(interp, argc, args, 2, 
+	    Tix_ArgcError(interp, argc, args, 2, 
 		"page ?option value ...?");
 	    goto error;
 	}
@@ -710,11 +699,11 @@ WidgetCommand(clientData, interp, argc, args)
     }
 
   done:
-    Tk_Release((ClientData) wPtr);
+    Tcl_Release((ClientData) wPtr);
     return result;
 
   error:
-    Tk_Release((ClientData) wPtr);
+    Tcl_Release((ClientData) wPtr);
     return TCL_ERROR;
 }
 
@@ -738,7 +727,6 @@ WidgetCommand(clientData, interp, argc, args)
  *
  *----------------------------------------------------------------------
  */
-
 static int
 WidgetConfigure(interp, wPtr, argc, args, flags)
     Tcl_Interp *interp;			/* Used for error reporting. */
@@ -783,7 +771,7 @@ WidgetConfigure(interp, wPtr, argc, args, flags)
      */
     gcValues.foreground = wPtr->textColorPtr->pixel;
     gcValues.background = Tk_3DBorderColor(wPtr->bgBorder)->pixel;
-    gcValues.font = wPtr->fontPtr->fid;
+    gcValues.font = TixFontId(wPtr->font);
     gcValues.graphics_exposures = False;
     newGC = Tk_GetGC(wPtr->tkwin,
 	 GCBackground|GCForeground|GCFont|GCGraphicsExposures,
@@ -813,7 +801,7 @@ WidgetConfigure(interp, wPtr, argc, args, flags)
 	gcValues.stipple = wPtr->gray;
 	mask = GCForeground|GCFillStyle|GCFont|GCStipple;
     }
-    gcValues.font = wPtr->fontPtr->fid;
+    gcValues.font = TixFontId(wPtr->font);
     newGC = Tk_GetGC(wPtr->tkwin, mask, &gcValues);
     if (wPtr->disabledGC != None) {
 	Tk_FreeGC(wPtr->display, wPtr->disabledGC);
@@ -837,6 +825,7 @@ WidgetConfigure(interp, wPtr, argc, args, flags)
     wPtr->focusGC = newGC;
 
     WidgetComputeGeometry(wPtr);
+    RedrawWhenIdle(wPtr);
 
     return TCL_OK;
 }
@@ -874,7 +863,7 @@ WidgetEventProc(clientData, eventPtr)
 
 	}
 	CancelRedrawWhenIdle(wPtr);
-	Tk_EventuallyFree((ClientData) wPtr, WidgetDestroy);
+	Tcl_EventuallyFree((ClientData) wPtr, WidgetDestroy);
 	break;
 
       case Expose:
@@ -906,7 +895,7 @@ WidgetEventProc(clientData, eventPtr)
  *
  * WidgetDestroy --
  *
- *	This procedure is invoked by Tk_EventuallyFree or Tk_Release
+ *	This procedure is invoked by Tcl_EventuallyFree or Tcl_Release
  *	to clean up the internal structure of a Notebookframe at a safe time
  *	(when no-one is using it anymore).
  *
@@ -921,7 +910,7 @@ WidgetEventProc(clientData, eventPtr)
 
 static void
 WidgetDestroy(clientData)
-    ClientData clientData;	/* Info about my widget. */
+    char *clientData;	/* Info about my widget. */
 {
     WidgetPtr wPtr = (WidgetPtr) clientData;
     Tab * tPtr;
@@ -1045,7 +1034,7 @@ static int TabConfigure(wPtr, tPtr, args, argc)
 
     if (tPtr->text != NULL) {
 	tPtr->numChars = strlen(tPtr->text);
-	TkComputeTextGeometry(wPtr->fontPtr, tPtr->text, tPtr->numChars,
+	TixComputeTextGeometry(wPtr->font, tPtr->text, tPtr->numChars,
 	    tPtr->wrapLength, &tPtr->width, &tPtr->height);
     }
     else if (tPtr->image != NULL) {
@@ -1098,7 +1087,7 @@ AddTab(wPtr, name, args, argc)
     tPtr->next = NULL;
     tPtr->wPtr = wPtr;
     tPtr->name = (char*)strdup(name);
-    tPtr->state = tkNormalUid;
+    tPtr->state = tixNormalUid;
     tPtr->text = NULL;
     tPtr->width = 0;
     tPtr->height = 0;
@@ -1147,7 +1136,7 @@ static void DeleteTab(tPtr)
 
     if (tPtr->wPtr->tkwin) {
 	Tk_FreeOptions(tabConfigSpecs, (char *)tPtr, 
-	    Tk_Display(tPtr->wPtr->tkwin), 0);
+		       Tk_Display(tPtr->wPtr->tkwin), 0);
     }
     ckfree((char*)tPtr);
 }
@@ -1171,7 +1160,7 @@ RedrawWhenIdle(wPtr)
 {
     if (! wPtr->redrawing && Tk_IsMapped(wPtr->tkwin)) {
 	wPtr->redrawing = 1;
-	Tk_DoWhenIdle(WidgetDisplay, (ClientData)wPtr);
+	Tcl_DoWhenIdle(WidgetDisplay, (ClientData)wPtr);
     }
 }
 
@@ -1194,7 +1183,7 @@ CancelRedrawWhenIdle(wPtr)
 {
     if (wPtr->redrawing) {
 	wPtr->redrawing = 0;
-	Tk_CancelIdleCall(WidgetDisplay, (ClientData)wPtr);
+	Tcl_CancelIdleCall(WidgetDisplay, (ClientData)wPtr);
     }
 }
 
@@ -1232,12 +1221,12 @@ static void GetTabPoints(wPtr, tPtr, x, points)
  *
  *----------------------------------------------------------------------
  */
-static void DrawTab(wPtr, tPtr, x, isActive, pixmap)
+static void DrawTab(wPtr, tPtr, x, isActive, drawable)
     WidgetPtr wPtr;
     Tab * tPtr;
     int x;
     int isActive;
-    Pixmap pixmap;
+    Drawable drawable;
 {
     Tk_3DBorder border;
     XPoint points[NUM_TAB_POINTS];
@@ -1264,17 +1253,20 @@ static void DrawTab(wPtr, tPtr, x, isActive, pixmap)
 	    drawY += extraH/2;
 	    break;
 	  case TK_ANCHOR_N: case TK_ANCHOR_NE: case TK_ANCHOR_NW:
+	    /*
+	     * Do nothing.
+	     */
 	    break;
 	}
     }
 
-    Tk_Fill3DPolygon(wPtr->tkwin, pixmap,
+    Tk_Fill3DPolygon(wPtr->tkwin, drawable,
 	border, points, NUM_TAB_POINTS,
 	wPtr->borderWidth, TK_RELIEF_SUNKEN);
 
     if (tPtr->text != NULL) {
-	if (tPtr->state == tkNormalUid) {
-	    TkDisplayText(wPtr->display, pixmap, wPtr->fontPtr,
+	if (tPtr->state == tixNormalUid) {
+	    TixDisplayText(wPtr->display, drawable, wPtr->font,
 	        tPtr->text, tPtr->numChars,
 	    	drawX, drawY,
 	        tPtr->width,
@@ -1282,7 +1274,7 @@ static void DrawTab(wPtr, tPtr, x, isActive, pixmap)
 	        tPtr->underline,
 	        wPtr->textGC);
 	} else {
-	    TkDisplayText(wPtr->display, pixmap, wPtr->fontPtr,
+	    TixDisplayText(wPtr->display, drawable, wPtr->font,
 	        tPtr->text, tPtr->numChars,
 	    	drawX, drawY,
 	        tPtr->width,
@@ -1293,18 +1285,18 @@ static void DrawTab(wPtr, tPtr, x, isActive, pixmap)
     }
     else if (tPtr->image != NULL) {
 	Tk_RedrawImage(tPtr->image, 0, 0, tPtr->width, tPtr->height,
-	    pixmap, drawX, drawY);
+	    drawable, drawX, drawY);
     }
     else if (tPtr->bitmap != None) {
 	GC gc;
 
-	if (tPtr->state == tkNormalUid) {
+	if (tPtr->state == tixNormalUid) {
 	    gc = wPtr->textGC;
 	} else {
 	    gc = wPtr->disabledGC;
 	}
 	XSetClipOrigin(wPtr->display, gc, drawX, drawY);
-	XCopyPlane(wPtr->display, tPtr->bitmap, pixmap,
+	XCopyPlane(wPtr->display, tPtr->bitmap, drawable,
 		gc, 0, 0, tPtr->width, tPtr->height,
 		drawX, drawY, 1);
 	XSetClipOrigin(wPtr->display, gc, 0, 0);
@@ -1312,7 +1304,7 @@ static void DrawTab(wPtr, tPtr, x, isActive, pixmap)
 
 #if 0
     if (wPtr->gotFocus && tPtr == wPtr->focus) {
-	XDrawLine(Tk_Display(wPtr->tkwin), pixmap, wPtr->focusGC,
+	XDrawLine(Tk_Display(wPtr->tkwin), drawable, wPtr->focusGC,
             drawX,
 	    drawY + tPtr->height + 1, 
 	    drawX + tPtr->width,
@@ -1333,11 +1325,11 @@ static void DrawTab(wPtr, tPtr, x, isActive, pixmap)
  *
  *----------------------------------------------------------------------
  */
-static void FocusTab(wPtr, tPtr, x, pixmap)
+static void FocusTab(wPtr, tPtr, x, drawable)
     WidgetPtr wPtr;
     Tab * tPtr;
     int x;
-    Pixmap pixmap;
+    Drawable drawable;
 {
     Tk_3DBorder border;
 
@@ -1350,12 +1342,12 @@ static void FocusTab(wPtr, tPtr, x, pixmap)
     }
 
     GetTabPoints(wPtr, tPtr, x, points);
-    Tk_Draw3DPolygon(wPtr->tkwin, pixmap,
+    Tk_Draw3DPolygon(wPtr->tkwin, drawable,
 	wPtr->focusBorder, points, NUM_TAB_POINTS,
 	wPtr->borderWidth, TK_RELIEF_SUNKEN);
 
     if (wPtr->active == tPtr) {
-	Tk_Draw3DPolygon(wPtr->tkwin, pixmap,
+	Tk_Draw3DPolygon(wPtr->tkwin, drawable,
 	    border, points, NUM_TAB_POINTS,
 	    wPtr->borderWidth/2, TK_RELIEF_SUNKEN);
     }
@@ -1381,7 +1373,7 @@ WidgetDisplay(clientData)
     WidgetPtr wPtr = (WidgetPtr) clientData;
     Tab * tPtr;
     int width, height;
-    Pixmap pixmap;
+    Drawable buffer;
 
     /* Now let's redraw */
     if (wPtr->tabHead == NULL) {
@@ -1398,20 +1390,13 @@ WidgetDisplay(clientData)
     else {
 	int x, y, activex = 0;
 
-	/*
-	 * Redrawing is done in a temporary pixmap that is allocated
-	 * here and freed at the end of the procedure.  All drawing is
-	 * done to the pixmap, and the pixmap is copied to the screen
-	 * at the end of the procedure.  This provides the smoothest
-	 * possible visual effects (no flashing on the screen).
-	 */
-	pixmap = Tk_GetPixmap(wPtr->display, Tk_WindowId(wPtr->tkwin),
+	buffer = Tix_GetRenderBuffer(wPtr->display, Tk_WindowId(wPtr->tkwin),
 	    Tk_Width(wPtr->tkwin), Tk_Height(wPtr->tkwin),
 	    Tk_Depth(wPtr->tkwin));
-	XFillRectangle(Tk_Display(wPtr->tkwin), pixmap, wPtr->backPageGC,
+	XFillRectangle(Tk_Display(wPtr->tkwin), buffer, wPtr->backPageGC,
 	    0, 0, Tk_Width(wPtr->tkwin), Tk_Height(wPtr->tkwin));
 
-	Tk_Fill3DRectangle(wPtr->tkwin, pixmap,
+	Tk_Fill3DRectangle(wPtr->tkwin, buffer,
 		wPtr->bgBorder, 0, wPtr->tabsHeight,
 		Tk_Width(wPtr->tkwin), 
 		Tk_Height(wPtr->tkwin) - wPtr->tabsHeight,
@@ -1422,13 +1407,13 @@ WidgetDisplay(clientData)
 	for (tPtr=wPtr->tabHead; tPtr; tPtr=tPtr->next) {
 	    if (tPtr == wPtr->active) {
 		activex = x;
-		DrawTab(wPtr, tPtr, x, 1, pixmap);
+		DrawTab(wPtr, tPtr, x, 1, buffer);
 	    }
 	    else {
-		DrawTab(wPtr, tPtr, x, 0, pixmap);
+		DrawTab(wPtr, tPtr, x, 0, buffer);
 	    }
 	    if (tPtr == wPtr->focus && wPtr->gotFocus) {
-		FocusTab(wPtr, tPtr, x, pixmap);
+		FocusTab(wPtr, tPtr, x, buffer);
 	    }
 
 	    x += (wPtr->borderWidth +  wPtr->tabPadx) * 2;
@@ -1436,7 +1421,7 @@ WidgetDisplay(clientData)
 	}
 
 	/* Draw the box */
-	Tk_Draw3DRectangle(wPtr->tkwin, pixmap,
+	Tk_Draw3DRectangle(wPtr->tkwin, buffer,
 		wPtr->bgBorder, 0, wPtr->tabsHeight,
 		Tk_Width(wPtr->tkwin), 
 		Tk_Height(wPtr->tkwin) - wPtr->tabsHeight,
@@ -1451,15 +1436,17 @@ WidgetDisplay(clientData)
 	    height = wPtr->borderWidth;
 	    width = wPtr->active->width + wPtr->tabPadx*2;
 
-	    XFillRectangle(wPtr->display, pixmap,
+	    XFillRectangle(wPtr->display, buffer,
 		Tk_3DBorderGC(wPtr->tkwin, wPtr->bgBorder, TK_3D_FLAT_GC),
 		x, y, width, height);
 	}
 
-	XCopyArea(wPtr->display, pixmap, Tk_WindowId(wPtr->tkwin),
-	    wPtr->textGC, 0, 0, Tk_Width(wPtr->tkwin), Tk_Height(wPtr->tkwin),
-	    0, 0);
-	Tk_FreePixmap(wPtr->display, pixmap);
+	if (buffer != Tk_WindowId(wPtr->tkwin)) {
+	    XCopyArea(wPtr->display, buffer, Tk_WindowId(wPtr->tkwin),
+	        wPtr->textGC, 0, 0, Tk_Width(wPtr->tkwin),
+		Tk_Height(wPtr->tkwin), 0, 0);
+	    Tk_FreePixmap(wPtr->display, buffer);
+	}
     }
 
     wPtr->redrawing = 0;
@@ -1499,7 +1486,7 @@ ComputeGeometry(wPtr)
 	    
 	    if (tPtr->text != NULL) {
 		tPtr->numChars = strlen(tPtr->text);
-		TkComputeTextGeometry(wPtr->fontPtr, tPtr->text,
+		TixComputeTextGeometry(wPtr->font, tPtr->text,
 		    tPtr->numChars, tPtr->wrapLength, 
 		    &tPtr->width, &tPtr->height);
 	    }
@@ -1595,4 +1582,5 @@ ImageProc(clientData, x, y, width, height, imgWidth, imgHeight)
     tPtr = (Tab *)clientData;
 
     WidgetComputeGeometry(tPtr->wPtr);
+    RedrawWhenIdle(tPtr->wPtr);
 }
