@@ -8,11 +8,10 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
+ *
+ * SCCS: @(#) tkCanvArc.c 1.34 97/04/25 16:50:56
  */
 
-static char sccsid[] = "@(#) tkCanvArc.c 1.26 95/06/07 22:53:15";
-
-#include <stdio.h>
 #include "tkPort.h"
 #include "tkInt.h"
 #include "tkCanvases.h"
@@ -69,6 +68,10 @@ typedef struct ArcItem  {
  * Information used for parsing configuration specs:
  */
 
+static Tk_CustomOption tagsOption = {Tk_CanvasTagsParseProc,
+    Tk_CanvasTagsPrintProc, (ClientData) NULL
+};
+
 static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_DOUBLE, "-extent",          NULL,          NULL,
 	"90", Tk_Offset(ArcItem, extent), TK_CONFIG_DONT_SET_DEFAULT},
@@ -85,7 +88,7 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_UID, "-style",          NULL,          NULL,
 	"pieslice", Tk_Offset(ArcItem, style), TK_CONFIG_DONT_SET_DEFAULT},
     {TK_CONFIG_CUSTOM, "-tags",          NULL,          NULL,
-	         NULL, 0, TK_CONFIG_NULL_OK, &tk_CanvasTagsOption},
+	         NULL, 0, TK_CONFIG_NULL_OK, &tagsOption},
     {TK_CONFIG_PIXELS, "-width",          NULL,          NULL,
 	"1", Tk_Offset(ArcItem, width), TK_CONFIG_DONT_SET_DEFAULT},
     {TK_CONFIG_END,          NULL,          NULL,          NULL,
@@ -207,9 +210,9 @@ CreateArc(interp, canvas, itemPtr, argc, args)
     ArcItem *arcPtr = (ArcItem *) itemPtr;
 
     if (argc < 4) {
-	Tcl_AppendResult(interp, "wrong # args:  should be \"",
-		Tk_PathName(Tk_CanvasTkwin(canvas)), "\" create ",
-		itemPtr->typePtr->name, " x1 y1 x2 y2 ?options?",
+	Tcl_AppendResult(interp, "wrong # args: should be \"",
+		Tk_PathName(Tk_CanvasTkwin(canvas)), " create ",
+		itemPtr->typePtr->name, " x1 y1 x2 y2 ?options?\"",
 		         NULL);
 	return TCL_ERROR;
     }
@@ -314,7 +317,7 @@ ArcCoords(interp, canvas, itemPtr, argc, args)
 	ComputeArcBbox(canvas, arcPtr);
     } else {
 	Tcl_SprintfResult(interp,
-		"wrong # coordinates:  expected 0 or 4, got %d",
+		"wrong # coordinates: expected 0 or 4, got %d",
 		argc);
 	return TCL_ERROR;
     }
@@ -367,12 +370,12 @@ ConfigureArc(interp, canvas, itemPtr, argc, args, flags)
      * style and graphics contexts.
      */
 
-    i = arcPtr->start/360.0;
+    i = (int) (arcPtr->start/360.0);
     arcPtr->start -= i*360.0;
     if (arcPtr->start < 0) {
 	arcPtr->start += 360.0;
     }
-    i = arcPtr->extent/360.0;
+    i = (int) (arcPtr->extent/360.0);
     arcPtr->extent -= i*360.0;
 
     if ((arcPtr->style != arcUid) && (arcPtr->style != chordUid)
@@ -534,8 +537,8 @@ ComputeArcBbox(canvas, arcPtr)
      * 9-o'clock, and 12-o'clock positions, if they are relevant.
      */
 
-    arcPtr->header.x1 = arcPtr->header.x2 = arcPtr->center1[0];
-    arcPtr->header.y1 = arcPtr->header.y2 = arcPtr->center1[1];
+    arcPtr->header.x1 = arcPtr->header.x2 = (int) arcPtr->center1[0];
+    arcPtr->header.y1 = arcPtr->header.y2 = (int) arcPtr->center1[1];
     TkIncludePoint((Tk_Item *) arcPtr, arcPtr->center2);
     center[0] = (arcPtr->bbox[0] + arcPtr->bbox[2])/2;
     center[1] = (arcPtr->bbox[1] + arcPtr->bbox[3])/2;
@@ -590,10 +593,10 @@ ComputeArcBbox(canvas, arcPtr)
     } else {
 	tmp = (arcPtr->width + 1)/2 + 1;
     }
-    arcPtr->header.x1 -= tmp;
-    arcPtr->header.y1 -= tmp;
-    arcPtr->header.x2 += tmp;
-    arcPtr->header.y2 += tmp;
+    arcPtr->header.x1 -= (int) tmp;
+    arcPtr->header.y1 -= (int) tmp;
+    arcPtr->header.x2 += (int) tmp;
+    arcPtr->header.y2 += (int) tmp;
 }
 
 /*
@@ -643,8 +646,8 @@ DisplayArc(canvas, itemPtr, display, drawable, x, y, width, height)
     if (y2 <= y1) {
 	y2 = y1+1;
     }
-    start = (64*arcPtr->start) + 0.5;
-    extent = (64*arcPtr->extent) + 0.5;
+    start = (int) ((64*arcPtr->start) + 0.5);
+    extent = (int) ((64*arcPtr->extent) + 0.5);
 
     /*
      * Display filled arc first (if wanted), then outline.  If the extent
@@ -750,12 +753,6 @@ ArcToPoint(canvas, itemPtr, pointPtr)
     double poly[8], polyDist, width, t1, t2;
     int filled, angleInRange;
 
-    if ((arcPtr->fillGC != None) || (arcPtr->outlineGC == None)) {
-	filled = 1;
-    } else {
-	filled = 0;
-    }
-
     /*
      * See if the point is within the angular range of the arc.
      * Remember, X angles are backwards from the way we'd normally
@@ -957,7 +954,6 @@ ArcToArea(canvas, itemPtr, rectPtr)
      */
 
     pointPtr = points;
-    numPoints = 0;
     angle = -arcPtr->start*(PI/180.0);
     pointPtr[0] = rx*cos(angle);
     pointPtr[1] = ry*sin(angle);
@@ -1012,7 +1008,6 @@ ArcToArea(canvas, itemPtr, rectPtr)
 	pointPtr[0] = 0.0;
 	pointPtr[1] = ry;
 	numPoints++;
-	pointPtr += 2;
     }
 
     /*
