@@ -8,11 +8,10 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
+ *
+ * SCCS: @(#) tkCanvLine.c 1.46 97/04/25 16:51:02
  */
 
-static char sccsid[] = "@(#) tkCanvLine.c 1.36 95/10/11 14:07:14";
-
-#include <stdio.h>
 #include "tkPort.h"
 #include "tkInt.h"
 #include "tkCanvases.h"
@@ -122,6 +121,9 @@ static void		TranslateLine _ANSI_ARGS_((Tk_Canvas canvas,
 
 static Tk_CustomOption arrowShapeOption = {ParseArrowShape,
 	PrintArrowShape, (ClientData) NULL};
+static Tk_CustomOption tagsOption = {Tk_CanvasTagsParseProc,
+    Tk_CanvasTagsPrintProc, (ClientData) NULL
+};
 
 static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_UID, "-arrow",          NULL,          NULL,
@@ -142,7 +144,7 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_BITMAP, "-stipple",          NULL,          NULL,
 	         NULL, Tk_Offset(LineItem, fillStipple), TK_CONFIG_NULL_OK},
     {TK_CONFIG_CUSTOM, "-tags",          NULL,          NULL,
-	         NULL, 0, TK_CONFIG_NULL_OK, &tk_CanvasTagsOption},
+	         NULL, 0, TK_CONFIG_NULL_OK, &tagsOption},
     {TK_CONFIG_PIXELS, "-width",          NULL,          NULL,
 	"1", Tk_Offset(LineItem, width), TK_CONFIG_DONT_SET_DEFAULT},
     {TK_CONFIG_END,          NULL,          NULL,          NULL,
@@ -227,9 +229,9 @@ CreateLine(interp, canvas, itemPtr, argc, args)
     int i;
 
     if (argc < 4) {
-	Tcl_AppendResult(interp, "wrong # args:  should be \"",
-		Tk_PathName(Tk_CanvasTkwin(canvas)), "\" create ",
-		itemPtr->typePtr->name, " x1 y1 x2 y2 ?x3 y3 ...? ?options?",
+	Tcl_AppendResult(interp, "wrong # args: should be \"",
+		Tk_PathName(Tk_CanvasTkwin(canvas)), " create ",
+		itemPtr->typePtr->name, " x1 y1 x2 y2 ?x3 y3 ...? ?options?\"",
 		         NULL);
 	return TCL_ERROR;
     }
@@ -257,9 +259,9 @@ CreateLine(interp, canvas, itemPtr, argc, args)
 	bothUid = Tk_GetUid("both");
     }
     linePtr->arrow = noneUid;
-    linePtr->arrowShapeA = 8.0;
-    linePtr->arrowShapeB = 10.0;
-    linePtr->arrowShapeC = 3.0;
+    linePtr->arrowShapeA = (float)8.0;
+    linePtr->arrowShapeB = (float)10.0;
+    linePtr->arrowShapeC = (float)3.0;
     linePtr->firstArrowPtr = NULL;
     linePtr->lastArrowPtr = NULL;
     linePtr->smooth = 0;
@@ -344,7 +346,7 @@ LineCoords(interp, canvas, itemPtr, argc, args)
 	}
     } else if (argc < 4) {
 	Tcl_AppendResult(interp,
-		"too few coordinates for line:  must have at least 4",
+		"too few coordinates for line: must have at least 4",
 		         NULL);
 	return TCL_ERROR;
     } else if (argc & 1) {
@@ -595,11 +597,11 @@ ComputeLineBbox(canvas, linePtr)
 					 * recomputed. */
 {
     double *coordPtr;
-    int i;
+    int i, width;
 
     coordPtr = linePtr->coordPtr;
-    linePtr->header.x1 = linePtr->header.x2 = *coordPtr;
-    linePtr->header.y1 = linePtr->header.y2 = coordPtr[1];
+    linePtr->header.x1 = linePtr->header.x2 = (int) *coordPtr;
+    linePtr->header.y1 = linePtr->header.y2 = (int) coordPtr[1];
 
     /*
      * Compute the bounding box of all the points in the line,
@@ -615,10 +617,14 @@ ComputeLineBbox(canvas, linePtr)
 	    i++, coordPtr += 2) {
 	TkIncludePoint((Tk_Item *) linePtr, coordPtr);
     }
-    linePtr->header.x1 -= linePtr->width;
-    linePtr->header.x2 += linePtr->width;
-    linePtr->header.y1 -= linePtr->width;
-    linePtr->header.y2 += linePtr->width;
+    width = linePtr->width;
+    if (width < 1) {
+	width = 1;
+    }
+    linePtr->header.x1 -= width;
+    linePtr->header.x2 += width;
+    linePtr->header.y1 -= width;
+    linePtr->header.y2 += width;
 
     /*
      * For mitered lines, make a second pass through all the points.
@@ -633,7 +639,7 @@ ComputeLineBbox(canvas, linePtr)
 	    int j;
     
 	    if (TkGetMiterPoints(coordPtr, coordPtr+2, coordPtr+4,
-		    (double) linePtr->width, miter, miter+2)) {
+		    (double) width, miter, miter+2)) {
 		for (j = 0; j < 4; j += 2) {
 		    TkIncludePoint((Tk_Item *) linePtr, miter+j);
 		}
@@ -1008,11 +1014,8 @@ LineToArea(canvas, itemPtr, rectPtr)
     LineItem *linePtr = (LineItem *) itemPtr;
     double staticSpace[2*MAX_STATIC_POINTS];
     double *linePoints;
-    double radius;
     int numPoints, result;
 
-    radius = linePtr->width/2.0;
-
     /*
      * Handle smoothed lines by generating an expanded set of points
      * against which to do the check.
@@ -1248,9 +1251,9 @@ ParseArrowShape(clientData, interp, tkwin, value, recordPtr, offset)
 		!= TCL_OK)) {
 	goto syntaxError;
     }
-    linePtr->arrowShapeA = a;
-    linePtr->arrowShapeB = b;
-    linePtr->arrowShapeC = c;
+    linePtr->arrowShapeA = (float) a;
+    linePtr->arrowShapeB = (float) b;
+    linePtr->arrowShapeC = (float) c;
     if (freeProc)
      (*freeProc)(argc,args);
     return TCL_OK;
