@@ -11,14 +11,15 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
+ *
+ * SCCS: @(#) tkFrame.c 1.82 97/08/08 17:26:26
  */
 
-static char sccsid[] = "@(#) tkFrame.c 1.59 95/06/04 14:08:47";
-
 #include "default.h"
 #include "tkPort.h"
 #include "tkInt.h"
-#include "tkVMacro.h"
+#include "tkOption.h"
+#include "tkOption.m"
 
 /*
  * A data structure of the following type is kept for each
@@ -44,12 +45,14 @@ typedef struct {
     char *screenName;		/* Screen on which widget is created.  Non-null
 				 * only for top-levels.  Malloc-ed, may be
 				 * NULL. */
-    char *visualName;		/* Textual description of visual for window,
+    Arg visualName;		/* Textual description of visual for window,
 				 * from -visual option.  Malloc-ed, may be
 				 * NULL. */
     char *colormapName;		/* Textual description of colormap for window,
 				 * from -colormap option.  Malloc-ed, may be
 				 * NULL. */
+    Arg menuName;		/* Textual description of menu to use for
+				 * menubar. Malloc-ed, may be NULL. */
     Colormap colormap;		/* If not None, identifies a colormap
 				 * allocated for this window, which must be
 				 * freed when the window is deleted. */
@@ -69,10 +72,16 @@ typedef struct {
 				 * don't request any size. */
     int height;			/* Height to request for window.  <= 0 means
 				 * don't request any size. */
-    Cursor cursor;		/* Current cursor for window, or None. */
+    Tk_Cursor cursor;		/* Current cursor for window, or None. */
     char *takeFocus;		/* Value of -takefocus option;  not used in
 				 * the C code, but used by keyboard traversal
 				 * scripts.  Malloc'ed, but may be NULL. */
+    int isContainer;		/* 1 means this window is a container, 0 means
+				 * that it isn't. */
+    Arg useThis;		/* If the window is embedded, this points to
+				 * the name of the window in which it is
+				 * embedded (malloc'ed).  For non-embedded
+				 * windows this is NULL. */
     int flags;			/* Various flags;  see below for
 				 * definitions. */
 } Frame;
@@ -83,13 +92,11 @@ typedef struct {
  * REDRAW_PENDING:		Non-zero means a DoWhenIdle handler
  *				has already been queued to redraw
  *				this window.
- * CLEAR_NEEDED;		Need to clear the window when redrawing.
  * GOT_FOCUS:			Non-zero means this widget currently
  *				has the input focus.
  */
 
 #define REDRAW_PENDING		1
-#define CLEAR_NEEDED		2
 #define GOT_FOCUS		4
 
 /*
@@ -121,6 +128,8 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_STRING, "-colormap", "colormap", "Colormap",
 	DEF_FRAME_COLORMAP, Tk_Offset(Frame, colormapName),
 	BOTH|TK_CONFIG_NULL_OK},
+    {TK_CONFIG_BOOLEAN, "-container", "container", "Container",
+	DEF_FRAME_CONTAINER, Tk_Offset(Frame, isContainer), BOTH},
     {TK_CONFIG_ACTIVE_CURSOR, "-cursor", "cursor", "Cursor",
 	DEF_FRAME_CURSOR, Tk_Offset(Frame, cursor), BOTH|TK_CONFIG_NULL_OK},
     {TK_CONFIG_PIXELS, "-height", "height", "Height",
@@ -133,6 +142,9 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_PIXELS, "-highlightthickness", "highlightThickness",
 	"HighlightThickness",
 	DEF_FRAME_HIGHLIGHT_WIDTH, Tk_Offset(Frame, highlightWidth), BOTH},
+    {TK_CONFIG_LANGARG, "-menu", "menu", "Menu",
+	DEF_TOPLEVEL_MENU, Tk_Offset(Frame, menuName),
+	TOPLEVEL|TK_CONFIG_NULL_OK},
     {TK_CONFIG_RELIEF, "-relief", "relief", "Relief",
 	DEF_FRAME_RELIEF, Tk_Offset(Frame, relief), BOTH},
     {TK_CONFIG_STRING, "-screen", "screen", "Screen",
@@ -141,7 +153,9 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_STRING, "-takefocus", "takeFocus", "TakeFocus",
 	DEF_FRAME_TAKE_FOCUS, Tk_Offset(Frame, takeFocus),
 	BOTH|TK_CONFIG_NULL_OK},
-    {TK_CONFIG_STRING, "-visual", "visual", "Visual",
+    {TK_CONFIG_LANGARG, "-use", "use", "Use",
+	DEF_FRAME_USE, Tk_Offset(Frame, useThis), TOPLEVEL|TK_CONFIG_NULL_OK},
+    {TK_CONFIG_LANGARG, "-visual", "visual", "Visual",
 	DEF_FRAME_VISUAL, Tk_Offset(Frame, visualName),
 	BOTH|TK_CONFIG_NULL_OK},
     {TK_CONFIG_PIXELS, "-width", "width", "Width",
@@ -157,7 +171,7 @@ static Tk_ConfigSpec configSpecs[] = {
 static int		ConfigureFrame _ANSI_ARGS_((Tcl_Interp *interp,
 			    Frame *framePtr, int argc, Arg *args,
 			    int flags));
-static void		DestroyFrame _ANSI_ARGS_((ClientData clientData));
+static void		DestroyFrame _ANSI_ARGS_((char *memPtr));
 static void		DisplayFrame _ANSI_ARGS_((ClientData clientData));
 static void		FrameCmdDeletedProc _ANSI_ARGS_((
 			    ClientData clientData));
@@ -170,17 +184,18 @@ static void		MapFrame _ANSI_ARGS_((ClientData clientData));
 /*
  *--------------------------------------------------------------
  *
- * Tk_FrameCmd --
+ * Tk_FrameCmd, Tk_ToplevelCmd --
  *
- *	This procedure is invoked to process the "frame" and
+ *	These procedures are invoked to process the "frame" and
  *	"toplevel" Tcl commands.  See the user documentation for
- *	details on what it does.
+ *	details on what they do.
  *
  * Results:
  *	A standard Tcl result.
  *
  * Side effects:
- *	See the user documentation.
+ *	See the user documentation.  These procedures are just wrappers;
+ *	they call ButtonCreate to do all of the real work.
  *
  *--------------------------------------------------------------
  */
@@ -192,15 +207,64 @@ Tk_FrameCmd(clientData, interp, argc, args)
     Tcl_Interp *interp;		/* Current interpreter. */
     int argc;			/* Number of arguments. */
     Arg *args;		/* Argument strings. */
+{
+    return TkCreateFrame(clientData, interp, argc, args, 0,          NULL);
+}
+
+int
+Tk_ToplevelCmd(clientData, interp, argc, args)
+    ClientData clientData;	/* Main window associated with
+				 * interpreter. */
+    Tcl_Interp *interp;		/* Current interpreter. */
+    int argc;			/* Number of arguments. */
+    Arg *args;		/* Argument strings. */
+{
+    return TkCreateFrame(clientData, interp, argc, args, 1,          NULL);
+}
+
+/*
+ *--------------------------------------------------------------
+ *
+ * TkFrameCreate --
+ *
+ *	This procedure is invoked to process the "frame" and "toplevel"
+ *	Tcl commands;  it is also invoked directly by Tk_Init to create
+ *	a new main window.  See the user documentation for the "frame"
+ *	and "toplevel" commands for details on what it does.
+ *
+ * Results:
+ *	A standard Tcl result.
+ *
+ * Side effects:
+ *	See the user documentation.
+ *
+ *--------------------------------------------------------------
+ */
+
+int
+TkCreateFrame(clientData, interp, argc, args, toplevel, appName)
+    ClientData clientData;	/* Main window associated with interpreter.
+				 * If we're called by Tk_Init to create a
+				 * new application, then this is NULL. */
+    Tcl_Interp *interp;		/* Current interpreter. */
+    int argc;			/* Number of arguments. */
+    Arg *args;		/* Argument strings. */
+    int toplevel;		/* Non-zero means create a toplevel window,
+				 * zero means create a frame. */
+    char *appName;		/* Should only be non-NULL if clientData is
+				 * NULL:  gives the base name to use for the
+				 * new application. */
 {
     Tk_Window tkwin = (Tk_Window) clientData;
     Frame *framePtr;
     Tk_Window new = NULL;
-    char *className, *screenName, *visualName, *colormapName, *arg;
-    int i, c, length, toplevel, depth;
+    char *className, *screenName, *colormapName, *arg;
+    Arg useOption = NULL;
+    Arg visualName;
+    int i, c, length, depth;
+    unsigned int mask;
     Colormap colormap;
     Visual *visual;
-    Display *display;
 
     if (argc < 2) {
 	Tcl_AppendResult(interp, "wrong # args: should be \"",
@@ -210,13 +274,14 @@ Tk_FrameCmd(clientData, interp, argc, args)
 
     /*
      * Pre-process the argument list.  Scan through it to find any
-     * "-class", "-screen", "-visual", and "-newcmap" options.  These
+     * "-class", "-screen", "-visual", and "-colormap" options.  These
      * arguments need to be processed specially, before the window
      * is configured using the usual Tk mechanisms.
      */
 
-    toplevel = (LangString(args[0])[0] == 't');
-    className = colormapName = screenName = visualName = NULL;
+    className = colormapName = screenName = NULL;
+    visualName = NULL;
+    colormap = None;
     for (i = 2; i < argc; i += 2) {
 	arg = LangString(args[i]);
 	length = strlen(arg);
@@ -233,23 +298,45 @@ Tk_FrameCmd(clientData, interp, argc, args)
 	} else if ((c == 's') && toplevel
 		&& (LangCmpOpt("-screen", arg, strlen(arg)) == 0)) {
 	    screenName = LangString(args[i+1]);
+	} else if ((c == 'u') && toplevel
+		&& (strncmp(arg, "-use", strlen(arg)) == 0)) {
+	    useOption = args[i+1];
 	} else if ((c == 'v') &&  LangCmpOpt("-visual",arg,strlen(arg)) == 0 ) {
 
-	    visualName = LangString(args[i+1]);
+	    visualName = args[i+1];
 	}
     }
 
     /*
-     * Create the window, and deal with the special options -classname,
-     * -colormap, -screenname, and -visual.  The order here is tricky,
-     * because we want to allow values for these options to come from
-     * the database, yet we can't do that until the window is created.
+     * Create the window, and deal with the special options -use,
+     * -classname, -colormap, -screenname, and -visual.  These options
+     * must be handle before calling ConfigureFrame below, and they must
+     * also be processed in a particular order, for the following
+     * reasons:
+     * 1. Must set the window's class before calling ConfigureFrame,
+     *    so that unspecified options are looked up in the option
+     *    database using the correct class.
+     * 2. Must set visual information before calling ConfigureFrame
+     *    so that colors are allocated in a proper colormap.
+     * 3. Must call TkpUseWindow before setting non-default visual
+     *    information, since TkpUseWindow changes the defaults.
      */
 
     if (screenName == NULL) {
 	screenName = (toplevel) ? "" : NULL;
     }
-    new = Tk_CreateWindowFromPath(interp, tkwin, LangString(args[1]), screenName);
+    if (tkwin != NULL) {
+	new = Tk_CreateWindowFromPath(interp, tkwin, LangString(args[1]), screenName);
+    } else {
+	/*
+	 * We were called from Tk_Init;  create a new application.
+	 */
+
+	if (appName == NULL) {
+	    panic("TkCreateFrame didn't get application name");
+	}
+	new = TkCreateMainWindow(interp, screenName, appName);
+    }
     if (new == NULL) {
 	goto error;
     }
@@ -260,13 +347,24 @@ Tk_FrameCmd(clientData, interp, argc, args)
 	}
     }
     Tk_SetClass(new, className);
+    if (useOption == NULL) {
+	char *s = Tk_GetOption(new, "use", "Use");
+        if (s)
+	    LangSetDefault(&useOption,s);
+    }
+    if (useOption != NULL) {
+	if (TkpUseWindow(interp, new, useOption) != TCL_OK) {
+	    goto error;
+	}
+    }
     if (visualName == NULL) {
-	visualName = Tk_GetOption(new, "visual", "Visual");
+	char *s = Tk_GetOption(new, "visual", "Visual");
+	if (s)
+	    LangSetDefault(&visualName,s);
     }
     if (colormapName == NULL) {
 	colormapName = Tk_GetOption(new, "colormap", "Colormap");
     }
-    colormap = None;
     if (visualName != NULL) {
 	visual = Tk_GetVisual(interp, new, visualName, &depth,
 		(colormapName == NULL) ? &colormap : (Colormap *) NULL);
@@ -283,72 +381,27 @@ Tk_FrameCmd(clientData, interp, argc, args)
 	Tk_SetWindowColormap(new, colormap);
     }
 
+    /*
+     * For top-level windows, provide an initial geometry request of
+     * 200x200,  just so the window looks nicer on the screen if it
+     * doesn't request a size for itself.
+     */
+
+    if (toplevel) {
+	Tk_GeometryRequest(new, 200, 200);
+    }
+
     /*
      * Create the widget record, process configuration options, and
      * create event handlers.  Then fill in a few additional fields
      * in the widget record from the special options.
      */
 
-    display = Tk_Display(new);
-    framePtr = (Frame *) TkInitFrame(interp, new, toplevel, argc-2, args+2);
-    if (framePtr == NULL) {
-	if (colormap != None) {
-	    Tk_FreeColormap(display, colormap);
-	}
-	return TCL_ERROR;
-    }
-    framePtr->colormap = colormap;
-    return TCL_OK;
-
-    error:
-    if (new != NULL) {
-	Tk_DestroyWindow(new);
-    }
-    return TCL_ERROR;
-}
-
-/*
- *----------------------------------------------------------------------
- *
- * TkInitFrame --
- *
- *	This procedure initializes a frame or toplevel widget.  It's
- *	separate from Tk_FrameCmd so that it can be used for the
- *	main window, which has already been created elsewhere.
- *
- * Results:
- *	Returns NULL if an error occurred while initializing the
- *	frame.  Otherwise returns a pointer to the frame's widget
- *	record (for use by Tk_FrameCmd, if it was the caller).
- *
- * Side effects:
- *	A widget record gets allocated, handlers get set up, etc..
- *
- *----------------------------------------------------------------------
- */
-
-char *
-TkInitFrame(interp, tkwin, toplevel, argc, args)
-    Tcl_Interp *interp;			/* Interpreter associated with the
-					 * application. */
-    Tk_Window tkwin;			/* Window to use for frame or
-					 * top-level.   Caller must already
-					 * have set window's class. */
-    int toplevel;			/* Non-zero means that this is a
-					 * top-level window, 0 means it's a
-					 * frame. */
-    int argc;				/* Number of configuration arguments
-					 * (not including class command and
-					 * window name). */
-    Arg *args;			/* Configuration arguments. */
-{
-    register Frame *framePtr;
-
     framePtr = (Frame *) ckalloc(sizeof(Frame));
-    framePtr->tkwin = tkwin;
-    framePtr->display = Tk_Display(tkwin);
+    framePtr->tkwin = new;
+    framePtr->display = Tk_Display(new);
     framePtr->interp = interp;
-    framePtr->widgetCmd = Lang_CreateWidget(interp,framePtr->tkwin, FrameWidgetCmd, (ClientData) framePtr, FrameCmdDeletedProc);
+    framePtr->widgetCmd = Lang_CreateWidget(interp,new, FrameWidgetCmd, (ClientData) framePtr, FrameCmdDeletedProc);
 
 
     framePtr->className = NULL;
@@ -356,7 +409,7 @@ TkInitFrame(interp, tkwin, toplevel, argc, args)
     framePtr->screenName = NULL;
     framePtr->visualName = NULL;
     framePtr->colormapName = NULL;
-    framePtr->colormap = None;
+    framePtr->colormap = colormap;
     framePtr->border = NULL;
     framePtr->borderWidth = 0;
     framePtr->relief = TK_RELIEF_FLAT;
@@ -367,19 +420,44 @@ TkInitFrame(interp, tkwin, toplevel, argc, args)
     framePtr->height = 0;
     framePtr->cursor = None;
     framePtr->takeFocus = NULL;
+    framePtr->isContainer = 0;
+    framePtr->useThis = NULL;
     framePtr->flags = 0;
-    Tk_CreateEventHandler(framePtr->tkwin,
-	    ExposureMask|StructureNotifyMask|FocusChangeMask,
-	    FrameEventProc, (ClientData) framePtr);
-    if (ConfigureFrame(interp, framePtr, argc, args, 0) != TCL_OK) {
-	Tk_DestroyWindow(framePtr->tkwin);
-	return NULL;
+    framePtr->menuName = NULL;
+
+    /*
+     * Store backreference to frame widget in window structure.
+     */
+    TkSetClassProcs(new, NULL, (ClientData) framePtr);
+
+    mask = ExposureMask | StructureNotifyMask | FocusChangeMask;
+    if (toplevel) {
+        mask |= ActivateMask;
+    }
+    Tk_CreateEventHandler(new, mask, FrameEventProc, (ClientData) framePtr);
+    if (ConfigureFrame(interp, framePtr, argc-2, args+2, 0) != TCL_OK) {
+	goto error;
+    }
+    if ((framePtr->isContainer)) {
+	if (framePtr->useThis == NULL) {
+	    TkpMakeContainer(framePtr->tkwin);
+	} else {
+	    Tcl_AppendResult(interp,"A window cannot have both the -use ",
+		    "and the -container option set.");
+	    return TCL_ERROR;
+	}
     }
     if (toplevel) {
-	Tk_DoWhenIdle(MapFrame, (ClientData) framePtr);
+	Tcl_DoWhenIdle(MapFrame, (ClientData) framePtr);
     }
-    Tcl_ArgResult(interp,LangWidgetArg(interp,framePtr->tkwin));
-    return (char *) framePtr;
+    Tcl_ArgResult(interp,LangWidgetArg(interp,new));
+    return TCL_OK;
+
+    error:
+    if (new != NULL) {
+	Tk_DestroyWindow(new);
+    }
+    return TCL_ERROR;
 }
 
 /*
@@ -408,7 +486,7 @@ FrameWidgetCmd(clientData, interp, argc, args)
     Arg *args;		/* Argument strings. */
 {
     register Frame *framePtr = (Frame *) clientData;
-    int result = TCL_OK;
+    int result;
     size_t length;
     int c, i;
 
@@ -417,7 +495,7 @@ FrameWidgetCmd(clientData, interp, argc, args)
 		LangString(args[0]), " option ?arg arg ...?\"",          NULL);
 	return TCL_ERROR;
     }
-    Tk_Preserve((ClientData) framePtr);
+    Tcl_Preserve((ClientData) framePtr);
     c = LangString(args[1])[0];
     length = strlen(LangString(args[1]));
     if ((c == 'c') && (strncmp(LangString(args[1]), "cget", length) == 0)
@@ -441,8 +519,8 @@ FrameWidgetCmd(clientData, interp, argc, args)
 		    (char *) framePtr, LangString(args[2]), framePtr->mask);
 	} else {
 	    /*
-	     * Don't allow the options -class, -newcmap, -screen,
-	     * or -visual to be changed.
+	     * Don't allow the options -class, -colormap, -container,
+	     * -newcmap, -screen, -use, or -visual to be changed.
 	     */
 
 	    for (i = 2; i < argc; i++) {
@@ -455,8 +533,12 @@ FrameWidgetCmd(clientData, interp, argc, args)
 			&& (length >= 2))
 			|| ((c == 'c') && (framePtr->mask == TOPLEVEL)
 			&& (LangCmpOpt("-colormap", LangString(args[i]), length) == 0))
+			|| ((c == 'c')
+			&& (LangCmpOpt("-container", LangString(args[i]), length) == 0))
 			|| ((c == 's') && (framePtr->mask == TOPLEVEL)
 			&& (LangCmpOpt("-screen", LangString(args[i]), length) == 0))
+			|| ((c == 'u') && (framePtr->mask == TOPLEVEL)
+			&& (LangCmpOpt("-use", LangString(args[i]), length) == 0))
 			|| ((c == 'v') && (framePtr->mask == TOPLEVEL)
 			&& (LangCmpOpt("-visual", LangString(args[i]), length) == 0))) {
 		    Tcl_AppendResult(interp, "can't modify ", LangString(args[i]),
@@ -470,12 +552,12 @@ FrameWidgetCmd(clientData, interp, argc, args)
 	}
     } else {
 	Tcl_AppendResult(interp, "bad option \"", LangString(args[1]),
-		"\":  must be cget or configure",          NULL);
+		"\": must be cget or configure",          NULL);
 	result = TCL_ERROR;
     }
 
     done:
-    Tk_Release((ClientData) framePtr);
+    Tcl_Release((ClientData) framePtr);
     return result;
 }
 
@@ -484,7 +566,7 @@ FrameWidgetCmd(clientData, interp, argc, args)
  *
  * DestroyFrame --
  *
- *	This procedure is invoked by Tk_EventuallyFree or Tk_Release
+ *	This procedure is invoked by Tcl_EventuallyFree or Tcl_Release
  *	to clean up the internal structure of a frame at a safe time
  *	(when no-one is using it anymore).
  *
@@ -498,10 +580,10 @@ FrameWidgetCmd(clientData, interp, argc, args)
  */
 
 static void
-DestroyFrame(clientData)
-    ClientData clientData;	/* Info about frame widget. */
+DestroyFrame(memPtr)
+    char *memPtr;		/* Info about frame widget. */
 {
-    register Frame *framePtr = (Frame *) clientData;
+    register Frame *framePtr = (Frame *) memPtr;
 
     Tk_FreeOptions(configSpecs, (char *) framePtr, framePtr->display,
 	    framePtr->mask);
@@ -541,14 +623,39 @@ ConfigureFrame(interp, framePtr, argc, args, flags)
     Arg *args;		/* Arguments. */
     int flags;			/* Flags to pass to Tk_ConfigureWidget. */
 {
+    Arg oldMenuName;
+    
+    /*
+     * Need the old menubar name for the menu code to delete it.
+     */
+    
+    if (framePtr->menuName == NULL) {
+    	oldMenuName = NULL;
+    } else {
+    	oldMenuName = LangCopyArg(framePtr->menuName);
+    }
+    
     if (Tk_ConfigureWidget(interp, framePtr->tkwin, configSpecs,
 	    argc, args, (char *) framePtr, flags | framePtr->mask) != TCL_OK) {
 	return TCL_ERROR;
     }
 
+    if (((oldMenuName == NULL) && (framePtr->menuName != NULL))
+	    || ((oldMenuName != NULL) && (framePtr->menuName == NULL))
+	    || ((oldMenuName != NULL) && (framePtr->menuName != NULL)
+	    && oldMenuName != framePtr->menuName 
+	    && strcmp(LangString(oldMenuName), LangString(framePtr->menuName)) != 0)) {
+	TkSetWindowMenuBar(interp, framePtr->tkwin, oldMenuName,
+		framePtr->menuName);
+    }
+        
+    /* FIXME: Suspicious */
     if (framePtr->border != NULL) {
 	Tk_SetBackgroundFromBorder(framePtr->tkwin, framePtr->border);
+    } else {
+	Tk_SetWindowBackgroundPixmap(framePtr->tkwin, None);
     }
+
     if (framePtr->highlightWidth < 0) {
 	framePtr->highlightWidth = 0;
     }
@@ -559,11 +666,15 @@ ConfigureFrame(interp, framePtr, argc, args, flags)
 		framePtr->height);
     }
 
+    if (oldMenuName != NULL) {
+    	LangFreeArg(oldMenuName, TCL_DYNAMIC);
+    }
+
     if (Tk_IsMapped(framePtr->tkwin)) {
 	if (!(framePtr->flags & REDRAW_PENDING)) {
-	    Tk_DoWhenIdle(DisplayFrame, (ClientData) framePtr);
+	    Tcl_DoWhenIdle(DisplayFrame, (ClientData) framePtr);
 	}
-	framePtr->flags |= REDRAW_PENDING|CLEAR_NEEDED;
+	framePtr->flags |= REDRAW_PENDING;
     }
     return TCL_OK;
 }
@@ -594,17 +705,14 @@ DisplayFrame(clientData)
     GC gc;
 
     framePtr->flags &= ~REDRAW_PENDING;
-    if ((framePtr->tkwin == NULL) || !Tk_IsMapped(tkwin)) {
+    /* FIXME: do we get here ? */
+    if ((framePtr->tkwin == NULL) || !Tk_IsMapped(tkwin)
+        || framePtr->isContainer) {
 	return;
     }
 
-    if (framePtr->flags & CLEAR_NEEDED) {
-	XClearWindow(framePtr->display, Tk_WindowId(tkwin));
-	framePtr->flags &= ~CLEAR_NEEDED;
-    }
-    if ((framePtr->border != NULL)
-	    && (framePtr->relief != TK_RELIEF_FLAT)) {
-	Tk_Draw3DRectangle(tkwin, Tk_WindowId(tkwin),
+    if (framePtr->border != NULL) {
+	Tk_Fill3DRectangle(tkwin, Tk_WindowId(tkwin),
 		framePtr->border, framePtr->highlightWidth,
 		framePtr->highlightWidth,
 		Tk_Width(tkwin) - 2*framePtr->highlightWidth,
@@ -650,27 +758,39 @@ FrameEventProc(clientData, eventPtr)
 {
     register Frame *framePtr = (Frame *) clientData;
 
-    if ((eventPtr->type == Expose) && (eventPtr->xexpose.count == 0)) {
-	goto redraw;
-    } else if (eventPtr->type == ConfigureNotify) {
-	/*
-	 * If the window changed size then the borders need to be
-	 * redrawn.
-	 */
-
-	framePtr->flags |= CLEAR_NEEDED;
+    if (((eventPtr->type == Expose) && (eventPtr->xexpose.count == 0))
+	    || (eventPtr->type == ConfigureNotify)) {
 	goto redraw;
     } else if (eventPtr->type == DestroyNotify) {
+	if (framePtr->menuName != NULL) {
+	    TkSetWindowMenuBar(framePtr->interp, framePtr->tkwin,
+		    framePtr->menuName, NULL);
+	    LangFreeArg(framePtr->menuName,TCL_DYNAMIC);
+	    framePtr->menuName = NULL;
+	}
 	if (framePtr->tkwin != NULL) {
-	    framePtr->tkwin = NULL;
-	    Lang_DeleteWidget(framePtr->interp,framePtr->widgetCmd);
 
+	    /*
+	     * If this window is a container, then this event could be
+	     * coming from the embedded application, in which case
+	     * Tk_DestroyWindow hasn't been called yet.  When Tk_DestroyWindow
+	     * is called later, then another destroy event will be generated.
+	     * We need to be sure we ignore the second event, since the frame
+	     * could be gone by then.  To do so, delete the event handler
+	     * explicitly (normally it's done implicitly by Tk_DestroyWindow).
+	     */
+    
+	    Tk_DeleteEventHandler(framePtr->tkwin,
+		    ExposureMask|StructureNotifyMask|FocusChangeMask,
+		    FrameEventProc, (ClientData) framePtr);
+	    framePtr->tkwin = NULL;
+            Lang_DeleteWidget(framePtr->interp, framePtr->widgetCmd);
 	}
 	if (framePtr->flags & REDRAW_PENDING) {
-	    Tk_CancelIdleCall(DisplayFrame, (ClientData) framePtr);
+	    Tcl_CancelIdleCall(DisplayFrame, (ClientData) framePtr);
 	}
-	Tk_CancelIdleCall(MapFrame, (ClientData) framePtr);
-	Tk_EventuallyFree((ClientData) framePtr, DestroyFrame);
+	Tcl_CancelIdleCall(MapFrame, (ClientData) framePtr);
+	Tcl_EventuallyFree((ClientData) framePtr, DestroyFrame);
     } else if (eventPtr->type == FocusIn) {
 	if (eventPtr->xfocus.detail != NotifyInferior) {
 	    framePtr->flags |= GOT_FOCUS;
@@ -685,12 +805,15 @@ FrameEventProc(clientData, eventPtr)
 		goto redraw;
 	    }
 	}
+    } else if (eventPtr->type == ActivateNotify) {
+    	TkpSetMainMenubar(framePtr->interp, framePtr->tkwin,
+    		LangString(framePtr->menuName));
     }
     return;
 
     redraw:
     if ((framePtr->tkwin != NULL) && !(framePtr->flags & REDRAW_PENDING)) {
-	Tk_DoWhenIdle(DisplayFrame, (ClientData) framePtr);
+	Tcl_DoWhenIdle(DisplayFrame, (ClientData) framePtr);
 	framePtr->flags |= REDRAW_PENDING;
     }
 }
@@ -720,6 +843,13 @@ FrameCmdDeletedProc(clientData)
     Frame *framePtr = (Frame *) clientData;
     Tk_Window tkwin = framePtr->tkwin;
 
+    if (framePtr->menuName != NULL) {
+	TkSetWindowMenuBar(framePtr->interp, framePtr->tkwin,
+		framePtr->menuName, NULL);
+	LangFreeArg(framePtr->menuName,TCL_DYNAMIC);
+	framePtr->menuName = NULL;
+    }
+
     /*
      * This procedure could be invoked either because the window was
      * destroyed and the command was then deleted (in which case tkwin
@@ -763,9 +893,9 @@ MapFrame(clientData)
      * window manager doesn't get a false idea of its desired geometry.
      */
 
-    Tk_Preserve((ClientData) framePtr);
+    Tcl_Preserve((ClientData) framePtr);
     while (1) {
-	if (Tk_DoOneEvent(TK_IDLE_EVENTS) == 0) {
+	if (Tcl_DoOneEvent(TCL_IDLE_EVENTS) == 0) {
 	    break;
 	}
 
@@ -775,10 +905,44 @@ MapFrame(clientData)
 	 */
 
 	if (framePtr->tkwin == NULL) {
-	    Tk_Release((ClientData) framePtr);
+	    Tcl_Release((ClientData) framePtr);
 	    return;
 	}
     }
     Tk_MapWindow(framePtr->tkwin);
-    Tk_Release((ClientData) framePtr);
+    Tcl_Release((ClientData) framePtr);
+}
+
+/*
+ *--------------------------------------------------------------
+ *
+ * TkInstallFrameMenu --
+ *
+ *	This function is needed when a Windows HWND is created
+ *	and a menubar has been set to the window with a system
+ *	menu. It notifies the menu package so that the system
+ *	menu can be rebuilt.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	The system menu (if any) is created for the menubar
+ *	associated with this frame.
+ *
+ *--------------------------------------------------------------
+ */
+
+void
+TkInstallFrameMenu(tkwin)
+    Tk_Window tkwin;		/* The window that was just created. */
+{
+    TkWindow *winPtr = (TkWindow *) tkwin;
+
+    if (winPtr->mainPtr != NULL) {
+	Frame *framePtr;
+	framePtr = (Frame*) winPtr->instanceData;
+	TkpMenuNotifyToplevelCreate(winPtr->mainPtr->interp, 
+		LangString(framePtr->menuName));
+    }
 }
