@@ -4,14 +4,14 @@
  *	This procedure implements images of type "bitmap" for Tk.
  *
  * Copyright (c) 1994 The Regents of the University of California.
- * Copyright (c) 1994-1995 Sun Microsystems, Inc.
+ * Copyright (c) 1994-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
+ *
+ * SCCS: @(#) tkImgBmap.c 1.33 97/07/31 09:08:22
  */
-
-static char sccsid[] = "@(#) tkImgBmap.c 1.18 95/06/28 13:09:18";
-
+#define NEED_REAL_STDIO
 #include "tkInt.h"
 #include "tkPort.h"
 #include "tkVMacro.h"
@@ -78,8 +78,9 @@ typedef struct BitmapInstance {
  * The type record for bitmap images:
  */
 
+static int		GetByte _ANSI_ARGS_((Tcl_Channel chan));
 static int		ImgBmapCreate _ANSI_ARGS_((Tcl_Interp *interp,
-			    char *name, int argc, Arg *args,
+			    char *name, int argc, Tcl_Obj *objv[],
 			    Tk_ImageType *typePtr, Tk_ImageMaster master,
 			    ClientData *clientDataPtr));
 static ClientData	ImgBmapGet _ANSI_ARGS_((Tk_Window tkwin,
@@ -128,7 +129,7 @@ static Tk_ConfigSpec configSpecs[] = {
 /*
  * The following data structure is used to describe the state of
  * parsing a bitmap file or string.  It is used for communication
- * between GetBitmapData and NextBitmapWord.
+ * between TkGetBitmapData and NextBitmapWord.
  */
 
 #define MAX_WORD_LENGTH 100
@@ -136,7 +137,7 @@ typedef struct ParseInfo {
     char *string;		/* Next character of string data for bitmap,
 				 * or NULL if bitmap is being read from
 				 * file. */
-    FILE *f;			/* File containing bitmap data, or NULL
+    Tcl_Channel chan;		/* File containing bitmap data, or NULL
 				 * if no file. */
     char word[MAX_WORD_LENGTH+1];
 				/* Current word of bitmap data, NULL
@@ -148,9 +149,6 @@ typedef struct ParseInfo {
  * Prototypes for procedures used only locally in this file:
  */
 
-static char *		GetBitmapData _ANSI_ARGS_((Tcl_Interp *interp,
-			    char *string, char *fileName,
-			    int *widthPtr, int *heightPtr));
 static int		ImgBmapCmd _ANSI_ARGS_((ClientData clientData,
 			    Tcl_Interp *interp, int argc, Arg *args));
 static void		ImgBmapCmdDeletedProc _ANSI_ARGS_((
@@ -158,7 +156,7 @@ static void		ImgBmapCmdDeletedProc _ANSI_ARGS_((
 static void		ImgBmapConfigureInstance _ANSI_ARGS_((
 			    BitmapInstance *instancePtr));
 static int		ImgBmapConfigureMaster _ANSI_ARGS_((
-			    BitmapMaster *masterPtr, int argc, Arg *args,
+			    BitmapMaster *masterPtr, int argc, Tcl_Obj **args,
 			    int flags));
 static int		NextBitmapWord _ANSI_ARGS_((ParseInfo *parseInfoPtr));
 
@@ -181,12 +179,12 @@ static int		NextBitmapWord _ANSI_ARGS_((ParseInfo *parseInfoPtr));
 
 	/* ARGSUSED */
 static int
-ImgBmapCreate(interp, name, argc, args, typePtr, master, clientDataPtr)
+ImgBmapCreate(interp, name, argc, objv, typePtr, master, clientDataPtr)
     Tcl_Interp *interp;		/* Interpreter for application containing
 				 * image. */
     char *name;			/* Name to use for image. */
     int argc;			/* Number of arguments. */
-    Arg *args;		/* Argument strings for options (doesn't
+    Tcl_Obj *objv[];		/* Argument objects for options (doesn't
 				 * include image name or type). */
     Tk_ImageType *typePtr;	/* Pointer to our type record (not used). */
     Tk_ImageMaster master;	/* Token for image, to be used by us in
@@ -195,6 +193,7 @@ ImgBmapCreate(interp, name, argc, args, typePtr, master, clientDataPtr)
 				 * it will be returned in later callbacks. */
 {
     BitmapMaster *masterPtr;
+    int i;
 
     masterPtr = (BitmapMaster *) ckalloc(sizeof(BitmapMaster));
     masterPtr->tkMaster = master;
@@ -211,7 +210,8 @@ ImgBmapCreate(interp, name, argc, args, typePtr, master, clientDataPtr)
     masterPtr->maskFileString = NULL;
     masterPtr->maskDataString = NULL;
     masterPtr->instancePtr = NULL;
-    if (ImgBmapConfigureMaster(masterPtr, argc, args, 0) != TCL_OK) {
+
+    if (ImgBmapConfigureMaster(masterPtr, argc, objv, 0) != TCL_OK) {
 	ImgBmapDelete((ClientData) masterPtr);
 	return TCL_ERROR;
     }
@@ -244,12 +244,12 @@ ImgBmapConfigureMaster(masterPtr, argc, args, flags)
     BitmapMaster *masterPtr;	/* Pointer to data structure describing
 				 * overall bitmap image to (reconfigure). */
     int argc;			/* Number of entries in args. */
-    Arg *args;		/* Pairs of configuration options for image. */
+    Tcl_Obj **args;		/* Pairs of configuration options for image. */
     int flags;			/* Flags to pass to Tk_ConfigureWidget,
 				 * such as TK_CONFIG_ARGV_ONLY. */
 {
     BitmapInstance *instancePtr;
-    int maskWidth, maskHeight;
+    int maskWidth, maskHeight, dummy1, dummy2;
 
     if (Tk_ConfigureWidget(masterPtr->interp, Tk_MainWindow(masterPtr->interp),
 	    configSpecs, argc, args, (char *) masterPtr, flags)
@@ -267,9 +267,9 @@ ImgBmapConfigureMaster(masterPtr, argc, args, flags)
 	masterPtr->data = NULL;
     }
     if ((masterPtr->fileString != NULL) || (masterPtr->dataString != NULL)) {
-	masterPtr->data = GetBitmapData(masterPtr->interp,
+	masterPtr->data = TkGetBitmapData(masterPtr->interp,
 		masterPtr->dataString, masterPtr->fileString,
-		&masterPtr->width, &masterPtr->height);
+		&masterPtr->width, &masterPtr->height, &dummy1, &dummy2);
 	if (masterPtr->data == NULL) {
 	    return TCL_ERROR;
 	}
@@ -284,9 +284,9 @@ ImgBmapConfigureMaster(masterPtr, argc, args, flags)
 	    Tcl_SetResult(masterPtr->interp, "can't have mask without bitmap",TCL_STATIC);
 	    return TCL_ERROR;
 	}
-	masterPtr->maskData = GetBitmapData(masterPtr->interp,
+	masterPtr->maskData = TkGetBitmapData(masterPtr->interp,
 		masterPtr->maskDataString, masterPtr->maskFileString,
-		&maskWidth, &maskHeight);
+		&maskWidth, &maskHeight, &dummy1, &dummy2);
 	if (masterPtr->maskData == NULL) {
 	    return TCL_ERROR;
 	}
@@ -328,7 +328,7 @@ ImgBmapConfigureMaster(masterPtr, argc, args, flags)
  *	None.
  *
  * Side effects:
- *	Generates errors via Tk_BackgroundError if there are problems
+ *	Generates errors via Tcl_BackgroundError if there are problems
  *	in setting up the instance.
  *
  *----------------------------------------------------------------------
@@ -374,7 +374,7 @@ ImgBmapConfigureInstance(instancePtr)
     instancePtr->fg = colorPtr;
 
     if (instancePtr->bitmap != None) {
-	XFreePixmap(Tk_Display(instancePtr->tkwin), instancePtr->bitmap);
+	Tk_FreePixmap(Tk_Display(instancePtr->tkwin), instancePtr->bitmap);
 	instancePtr->bitmap = None;
     }
     if (masterPtr->data != NULL) {
@@ -386,7 +386,7 @@ ImgBmapConfigureInstance(instancePtr)
     }
 
     if (instancePtr->mask != None) {
-	XFreePixmap(Tk_Display(instancePtr->tkwin), instancePtr->mask);
+	Tk_FreePixmap(Tk_Display(instancePtr->tkwin), instancePtr->mask);
 	instancePtr->mask = None;
     }
     if (masterPtr->maskData != NULL) {
@@ -436,13 +436,13 @@ ImgBmapConfigureInstance(instancePtr)
     Tcl_AddErrorInfo(masterPtr->interp, "\n    (while configuring image \"");
     Tcl_AddErrorInfo(masterPtr->interp, Tk_NameOfImage(masterPtr->tkMaster));
     Tcl_AddErrorInfo(masterPtr->interp, "\")");
-    Tk_BackgroundError(masterPtr->interp);
+    Tcl_BackgroundError(masterPtr->interp);
 }
 
 /*
  *----------------------------------------------------------------------
  *
- * GetBitmapData --
+ * TkGetBitmapData --
  *
  *	Given a file name or ASCII string, this procedure parses the
  *	file or string contents to produce binary data for a bitmap.
@@ -450,7 +450,9 @@ ImgBmapConfigureInstance(instancePtr)
  * Results:
  *	If the bitmap description was parsed successfully then the
  *	return value is a malloc-ed array containing the bitmap data.
- *	The dimensions of the data are stored in *widthPtr and *heightPtr.
+ *	The dimensions of the data are stored in *widthPtr and
+ *	*heightPtr.  *hotXPtr and *hotYPtr are set to the bitmap
+ *	hotspot if one is defined, otherwise they are set to -1, -1.
  *	If an error occurred, NULL is returned and an error message is
  *	left in Tcl_GetResult(interp).
  *
@@ -460,8 +462,9 @@ ImgBmapConfigureInstance(instancePtr)
  *----------------------------------------------------------------------
  */
 
-static char *
-GetBitmapData(interp, string, fileName, widthPtr, heightPtr)
+char *
+TkGetBitmapData(interp, string, fileName, widthPtr, heightPtr,
+	hotXPtr, hotYPtr)
     Tcl_Interp *interp;			/* For reporting errors. */
     char *string;			/* String describing bitmap.  May
 					 * be NULL. */
@@ -471,8 +474,9 @@ GetBitmapData(interp, string, fileName, widthPtr, heightPtr)
 					 * string is NULL. */
     int *widthPtr, *heightPtr;		/* Dimensions of bitmap get returned
 					 * here. */
+    int *hotXPtr, *hotYPtr;		/* Position of hot spot or -1,-1. */
 {
-    int width, height, numBytes;
+    int width, height, numBytes, hotX, hotY;
     char *p, *end, *expandedFileName;
     ParseInfo pi;
     char *data = NULL;
@@ -480,19 +484,25 @@ GetBitmapData(interp, string, fileName, widthPtr, heightPtr)
 
     pi.string = string;
     if (string == NULL) {
-	expandedFileName = Tcl_TildeSubst(interp, fileName, &buffer);
+        if (Tcl_IsSafe(interp)) {
+            Tcl_AppendResult(interp, "can't get bitmap data from a file in a",
+                    " safe interpreter",          NULL);
+            return NULL;
+        }
+	expandedFileName = Tcl_TranslateFileName(interp, fileName, &buffer);
 	if (expandedFileName == NULL) {
 	    return NULL;
 	}
-	pi.f = fopen(expandedFileName, "r");
+	pi.chan = Tcl_OpenFileChannel(interp, expandedFileName, "r", 0);
 	Tcl_DStringFree(&buffer);
-	if (pi.f == NULL) {
+	if (pi.chan == NULL) {
+	    Tcl_ResetResult(interp);
 	    Tcl_AppendResult(interp, "couldn't read bitmap file \"",
 		    fileName, "\": ", Tcl_PosixError(interp),          NULL);
 	    return NULL;
 	}
     } else {
-	pi.f = NULL;
+	pi.chan = NULL;
     }
 
     /*
@@ -514,6 +524,8 @@ GetBitmapData(interp, string, fileName, widthPtr, heightPtr)
 
     width = 0;
     height = 0;
+    hotX = -1;
+    hotY = -1;
     while (1) {
 	if (NextBitmapWord(&pi) != TCL_OK) {
 	    goto error;
@@ -536,6 +548,24 @@ GetBitmapData(interp, string, fileName, widthPtr, heightPtr)
 	    if ((end == pi.word) || (*end != 0)) {
 		goto error;
 	    }
+	} else if ((pi.wordLength >= 6) && (pi.word[pi.wordLength-6] == '_')
+		&& (strcmp(pi.word+pi.wordLength-6, "_x_hot") == 0)) {
+	    if (NextBitmapWord(&pi) != TCL_OK) {
+		goto error;
+	    }
+	    hotX = strtol(pi.word, &end, 0);
+	    if ((end == pi.word) || (*end != 0)) {
+		goto error;
+	    }
+	} else if ((pi.wordLength >= 6) && (pi.word[pi.wordLength-6] == '_')
+		&& (strcmp(pi.word+pi.wordLength-6, "_y_hot") == 0)) {
+	    if (NextBitmapWord(&pi) != TCL_OK) {
+		goto error;
+	    }
+	    hotY = strtol(pi.word, &end, 0);
+	    if ((end == pi.word) || (*end != 0)) {
+		goto error;
+	    }
 	} else if ((pi.word[0] == 'c') && (strcmp(pi.word, "char") == 0)) {
 	    while (1) {
 		if (NextBitmapWord(&pi) != TCL_OK) {
@@ -568,7 +598,7 @@ GetBitmapData(interp, string, fileName, widthPtr, heightPtr)
 	if (NextBitmapWord(&pi) != TCL_OK) {
 	    goto error;
 	}
-	*p = strtol(pi.word, &end, 0);
+	*p = (char) strtol(pi.word, &end, 0);
 	if (end == pi.word) {
 	    goto error;
 	}
@@ -578,11 +608,13 @@ GetBitmapData(interp, string, fileName, widthPtr, heightPtr)
      * All done.  Clean up and return.
      */
 
-    if (pi.f != NULL) {
-	fclose(pi.f);
+    if (pi.chan != NULL) {
+	Tcl_Close(NULL, pi.chan);
     }
     *widthPtr = width;
     *heightPtr = height;
+    *hotXPtr = hotX;
+    *hotYPtr = hotY;
     return data;
 
     error:
@@ -591,8 +623,8 @@ GetBitmapData(interp, string, fileName, widthPtr, heightPtr)
     if (data != NULL) {
 	ckfree(data);
     }
-    if (pi.f != NULL) {
-	fclose(pi.f);
+    if (pi.chan != NULL) {
+	Tcl_Close(NULL, pi.chan);
     }
     return NULL;
 }
@@ -643,14 +675,14 @@ NextBitmapWord(parseInfoPtr)
 	}
 	parseInfoPtr->string = src;
     } else {
-	for (c = getc(parseInfoPtr->f); isspace(UCHAR(c)) || (c == ',');
-		c = getc(parseInfoPtr->f)) {
+	for (c = GetByte(parseInfoPtr->chan); isspace(UCHAR(c)) || (c == ',');
+		c = GetByte(parseInfoPtr->chan)) {
 	    if (c == EOF) {
 		return TCL_ERROR;
 	    }
 	}
 	for ( ; !isspace(UCHAR(c)) && (c != ',') && (c != EOF);
-		c = getc(parseInfoPtr->f)) {
+		c = GetByte(parseInfoPtr->chan)) {
 	    *dst = c;
 	    dst++;
 	    parseInfoPtr->wordLength++;
@@ -686,7 +718,7 @@ NextBitmapWord(parseInfoPtr)
 
 static int
 ImgBmapCmd(clientData, interp, argc, args)
-    ClientData clientData;	/* Information about button widget. */
+    ClientData clientData;	/* Information about the image master. */
     Tcl_Interp *interp;		/* Current interpreter. */
     int argc;			/* Number of arguments. */
     Arg *args;		/* Argument strings. */
@@ -909,10 +941,10 @@ ImgBmapFree(clientData, display)
 	Tk_FreeColor(instancePtr->bg);
     }
     if (instancePtr->bitmap != None) {
-	XFreePixmap(display, instancePtr->bitmap);
+	Tk_FreePixmap(display, instancePtr->bitmap);
     }
     if (instancePtr->mask != None) {
-	XFreePixmap(display, instancePtr->mask);
+	Tk_FreePixmap(display, instancePtr->mask);
     }
     if (instancePtr->gc != None) {
 	Tk_FreeGC(display, instancePtr->gc);
@@ -958,8 +990,7 @@ ImgBmapDelete(masterData)
     }
     masterPtr->tkMaster = NULL;
     if (masterPtr->imageCmd != NULL) {
-	Lang_DeleteObject(masterPtr->interp,masterPtr->imageCmd);
-
+	Lang_DeleteObject(masterPtr->interp, masterPtr->imageCmd);
     }
     if (masterPtr->data != NULL) {
 	ckfree(masterPtr->data);
@@ -1000,3 +1031,34 @@ ImgBmapCmdDeletedProc(clientData)
 	Tk_DeleteImage(masterPtr->interp, Tk_NameOfImage(masterPtr->tkMaster));
     }
 }
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * GetByte --
+ *
+ *	Get the next byte from the open channel.
+ *
+ * Results:
+ *	The next byte or EOF.
+ *
+ * Side effects:
+ *	We read from the channel.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static int
+GetByte(chan)
+    Tcl_Channel chan;	/* The channel we read from. */
+{
+    char buffer;
+    int size;
+
+    size = Tcl_Read(chan, &buffer, 1);
+    if (size <= 0) {
+	return EOF;
+    } else {
+	return buffer;
+    }
+}
