@@ -9,10 +9,10 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
+ *
+ * SCCS: @(#) tkPack.c 1.64 96/05/03 10:51:52
  */
 
-static char sccsid[] = "@(#) tkPack.c 1.56 95/11/24 17:52:11";
-
 #include "tkPort.h"
 #include "tkInt.h"
 
@@ -68,7 +68,7 @@ typedef struct Packer {
 /*
  * Flag values for Packer structures:
  *
- * REQUESTED_REPACK:		1 means a Tk_DoWhenIdle request
+ * REQUESTED_REPACK:		1 means a Tcl_DoWhenIdle request
  *				has already been made to repack
  *				all the slaves of this window.
  * FILLX:			1 means if frame allocated for window
@@ -117,6 +117,7 @@ static void		PackReqProc _ANSI_ARGS_((ClientData clientData,
 			    Tk_Window tkwin));
 static void		PackLostSlaveProc _ANSI_ARGS_((ClientData clientData,
 			    Tk_Window tkwin));
+static void		DestroyPacker _ANSI_ARGS_((char *memPtr));
 
 static Tk_GeomMgr packerType = {
     "pack",			/* name */
@@ -131,6 +132,7 @@ static Tk_GeomMgr packerType = {
 static void		ArrangePacking _ANSI_ARGS_((ClientData clientData));
 static int		ConfigureSlaves _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Window tkwin, int argc, Arg *args));
+static void             DestroyPacker _ANSI_ARGS_((char *memPtr));
 static Packer *		GetPacker _ANSI_ARGS_((Tk_Window tkwin));
 static int		PackAfter _ANSI_ARGS_((Tcl_Interp *interp,
 			    Packer *prevPtr, Packer *masterPtr, int argc,
@@ -368,7 +370,7 @@ Tk_PackCmd(clientData, interp, argc, args)
 	    }
 	    if (!(masterPtr->flags & REQUESTED_REPACK)) {
 		masterPtr->flags |= REQUESTED_REPACK;
-		Tk_DoWhenIdle(ArrangePacking, (ClientData) masterPtr);
+		Tcl_DoWhenIdle(ArrangePacking, (ClientData) masterPtr);
 	    }
 	} else {
 	    masterPtr->flags |= DONT_PROPAGATE;
@@ -417,7 +419,7 @@ Tk_PackCmd(clientData, interp, argc, args)
 	}
     } else {
 	Tcl_AppendResult(interp, "bad option \"", LangString(args[1]),
-		"\":  must be configure, forget, info, ",
+		"\": must be configure, forget, info, ",
 		"propagate, or slaves",          NULL);
 	return TCL_ERROR;
     }
@@ -456,7 +458,7 @@ PackReqProc(clientData, tkwin)
     packPtr = packPtr->masterPtr;
     if (!(packPtr->flags & REQUESTED_REPACK)) {
 	packPtr->flags |= REQUESTED_REPACK;
-	Tk_DoWhenIdle(ArrangePacking, (ClientData) packPtr);
+	Tcl_DoWhenIdle(ArrangePacking, (ClientData) packPtr);
     }
 }
 
@@ -498,7 +500,7 @@ PackLostSlaveProc(clientData, tkwin)
  *
  * ArrangePacking --
  *
- *	This procedure is invoked (using the Tk_DoWhenIdle
+ *	This procedure is invoked (using the Tcl_DoWhenIdle
  *	mechanism) to re-layout a set of windows managed by
  *	the packer.  It is invoked at idle time so that a
  *	series of packer requests can be merged into a single
@@ -559,7 +561,7 @@ ArrangePacking(clientData)
     }
     masterPtr->abortPtr = &abort;
     abort = 0;
-    Tk_Preserve((ClientData) masterPtr);
+    Tcl_Preserve((ClientData) masterPtr);
 
     /*
      * Pass #1: scan all the slaves to figure out the total amount
@@ -623,7 +625,7 @@ ArrangePacking(clientData)
 	    && !(masterPtr->flags & DONT_PROPAGATE)) {
 	Tk_GeometryRequest(masterPtr->tkwin, maxWidth, maxHeight);
 	masterPtr->flags |= REQUESTED_REPACK;
-	Tk_DoWhenIdle(ArrangePacking, (ClientData) masterPtr);
+	Tcl_DoWhenIdle(ArrangePacking, (ClientData) masterPtr);
 	goto done;
     }
 
@@ -808,7 +810,7 @@ ArrangePacking(clientData)
 
     done:
     masterPtr->abortPtr = NULL;
-    Tk_Release((ClientData) masterPtr);
+    Tcl_Release((ClientData) masterPtr);
 }
 
 /*
@@ -1132,7 +1134,7 @@ PackAfter(interp, prevPtr, masterPtr, argc, args)
 		    badPad:
 		    Tcl_AppendResult(interp, "bad pad value \"",
 			    options[index+1],
-			    "\":  must be positive screen distance",
+			    "\": must be positive screen distance",
 			             NULL);
 		    goto error;
 		}
@@ -1165,7 +1167,7 @@ PackAfter(interp, prevPtr, masterPtr, argc, args)
 		index++;
 	    } else {
 		Tcl_AppendResult(interp, "bad option \"", curOpt,
-			"\":  should be top, bottom, left, right, ",
+			"\": should be top, bottom, left, right, ",
 			"expand, fill, fillx, filly, padx, pady, or frame",
 			         NULL);
 		goto error;
@@ -1218,7 +1220,7 @@ PackAfter(interp, prevPtr, masterPtr, argc, args)
     }
     if (!(masterPtr->flags & REQUESTED_REPACK)) {
 	masterPtr->flags |= REQUESTED_REPACK;
-	Tk_DoWhenIdle(ArrangePacking, (ClientData) masterPtr);
+	Tcl_DoWhenIdle(ArrangePacking, (ClientData) masterPtr);
     }
     return TCL_OK;
 
@@ -1269,7 +1271,7 @@ Unlink(packPtr)
     }
     if (!(masterPtr->flags & REQUESTED_REPACK)) {
 	masterPtr->flags |= REQUESTED_REPACK;
-	Tk_DoWhenIdle(ArrangePacking, (ClientData) masterPtr);
+	Tcl_DoWhenIdle(ArrangePacking, (ClientData) masterPtr);
     }
     if (masterPtr->abortPtr != NULL) {
 	*masterPtr->abortPtr = 1;
@@ -1283,7 +1285,7 @@ Unlink(packPtr)
  *
  * DestroyPacker --
  *
- *	This procedure is invoked by Tk_EventuallyFree or Tk_Release
+ *	This procedure is invoked by Tcl_EventuallyFree or Tcl_Release
  *	to clean up the internal structure of a packer at a safe time
  *	(when no-one is using it anymore).
  *
@@ -1297,11 +1299,11 @@ Unlink(packPtr)
  */
 
 static void
-DestroyPacker(clientData)
-    ClientData clientData;		/* Info about packed window that
-					 * is now dead. */
+DestroyPacker(memPtr)
+    char *memPtr;		/* Info about packed window that
+				 * is now dead. */
 {
-    register Packer *packPtr = (Packer *) clientData;
+    register Packer *packPtr = (Packer *) memPtr;
     ckfree((char *) packPtr);
 }
 
@@ -1335,14 +1337,14 @@ PackStructureProc(clientData, eventPtr)
 	if ((packPtr->slavePtr != NULL)
 		&& !(packPtr->flags & REQUESTED_REPACK)) {
 	    packPtr->flags |= REQUESTED_REPACK;
-	    Tk_DoWhenIdle(ArrangePacking, (ClientData) packPtr);
+	    Tcl_DoWhenIdle(ArrangePacking, (ClientData) packPtr);
 	}
 	if (packPtr->doubleBw != 2*Tk_Changes(packPtr->tkwin)->border_width) {
 	    if ((packPtr->masterPtr != NULL)
 		    && !(packPtr->masterPtr->flags & REQUESTED_REPACK)) {
 		packPtr->doubleBw = 2*Tk_Changes(packPtr->tkwin)->border_width;
 		packPtr->masterPtr->flags |= REQUESTED_REPACK;
-		Tk_DoWhenIdle(ArrangePacking, (ClientData) packPtr->masterPtr);
+		Tcl_DoWhenIdle(ArrangePacking, (ClientData) packPtr->masterPtr);
 	    }
 	}
     } else if (eventPtr->type == DestroyNotify) {
@@ -1363,10 +1365,10 @@ PackStructureProc(clientData, eventPtr)
 	Tcl_DeleteHashEntry(Tcl_FindHashEntry(&packerHashTable,
 		(char *) packPtr->tkwin));
 	if (packPtr->flags & REQUESTED_REPACK) {
-	    Tk_CancelIdleCall(ArrangePacking, (ClientData) packPtr);
+	    Tcl_CancelIdleCall(ArrangePacking, (ClientData) packPtr);
 	}
 	packPtr->tkwin = NULL;
-	Tk_EventuallyFree((ClientData) packPtr, DestroyPacker);
+	Tcl_EventuallyFree((ClientData) packPtr, DestroyPacker);
     } else if (eventPtr->type == MapNotify) {
 	/*
 	 * When a master gets mapped, must redo the geometry computation
@@ -1376,7 +1378,7 @@ PackStructureProc(clientData, eventPtr)
 	if ((packPtr->slavePtr != NULL)
 		&& !(packPtr->flags & REQUESTED_REPACK)) {
 	    packPtr->flags |= REQUESTED_REPACK;
-	    Tk_DoWhenIdle(ArrangePacking, (ClientData) packPtr);
+	    Tcl_DoWhenIdle(ArrangePacking, (ClientData) packPtr);
 	}
     } else if (eventPtr->type == UnmapNotify) {
 	Packer *packPtr2;
@@ -1731,7 +1733,7 @@ ConfigureSlaves(interp, tkwin, argc, args)
 	}
 	if (!(masterPtr->flags & REQUESTED_REPACK)) {
 	    masterPtr->flags |= REQUESTED_REPACK;
-	    Tk_DoWhenIdle(ArrangePacking, (ClientData) masterPtr);
+	    Tcl_DoWhenIdle(ArrangePacking, (ClientData) masterPtr);
 	}
     }
     return TCL_OK;
