@@ -10,11 +10,9 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
+ *
+ * SCCS: @(#) tkTextWind.c 1.14 97/04/25 16:52:09
  */
-
-static char sccsid[] = "@(#) tkTextWind.c 1.10 95/04/30 16:27:13";
-
-#include "tkPort.h"
 #include "tk.h"
 #include "tkText.h"
 
@@ -159,7 +157,6 @@ TkTextWindowCmd(textPtr, interp, argc, args)
 				 * parsed this command enough to know that
 				 * LangString(args[1]) is "window". */
 {
-    int c;
     size_t length;
     register TkTextSegment *ewPtr;
 
@@ -168,7 +165,6 @@ TkTextWindowCmd(textPtr, interp, argc, args)
 		LangString(args[0]), " window option ?arg arg ...?\"",          NULL);
 	return TCL_ERROR;
     }
-    c = LangString(args[2])[0];
     length = strlen(LangString(args[2]));
     if ((strncmp(LangString(args[2]), "cget", length) == 0) && (length >= 2)) {
 	TkTextIndex index;
@@ -296,7 +292,7 @@ TkTextWindowCmd(textPtr, interp, argc, args)
 	}
     } else {
 	Tcl_AppendResult(interp, "bad window option \"", LangString(args[2]),
-		"\":  must be cget, configure, create, or names",
+		"\": must be cget, configure, create, or names",
 		         NULL);
 	return TCL_ERROR;
     }
@@ -622,7 +618,7 @@ EmbWinLostSlaveProc(clientData, tkwin)
 
     Tk_DeleteEventHandler(ewPtr->body.ew.tkwin, StructureNotifyMask,
 	    EmbWinStructureProc, (ClientData) ewPtr);
-    Tk_CancelIdleCall(EmbWinDelayedUnmap, (ClientData) ewPtr);
+    Tcl_CancelIdleCall(EmbWinDelayedUnmap, (ClientData) ewPtr);
     if (ewPtr->body.ew.textPtr->tkwin != Tk_Parent(tkwin)) {
 	Tk_UnmaintainGeometry(tkwin, ewPtr->body.ew.textPtr->tkwin);
     } else {
@@ -690,7 +686,7 @@ EmbWinDeleteProc(ewPtr, linePtr, treeGone)
 		EmbWinStructureProc, (ClientData) ewPtr);
 	Tk_DestroyWindow(ewPtr->body.ew.tkwin);
     }
-    Tk_CancelIdleCall(EmbWinDelayedUnmap, (ClientData) ewPtr);
+    Tcl_CancelIdleCall(EmbWinDelayedUnmap, (ClientData) ewPtr);
     Tk_FreeOptions(configSpecs, (char *) &ewPtr->body.ew,
 	    ewPtr->body.ew.textPtr->display, 0);
     ckfree((char *) ewPtr);
@@ -787,14 +783,15 @@ EmbWinLayoutProc(textPtr, indexPtr, ewPtr, offset, maxX, maxChars,
 	code = LangDoCallback(textPtr->interp, ewPtr->body.ew.create, 1, 0);
 	if (code != TCL_OK) {
 	    createError:
-	    Tk_BackgroundError(textPtr->interp);
-	    goto gotWindow;
+	    Tcl_BackgroundError(textPtr->interp);
+	    goto gotWindow;                   
 	}
-	Tcl_DStringInit(&name);
-	Tcl_DStringAppend(&name, Tcl_GetResult(textPtr->interp), -1);
+	Tcl_DStringInit(&name);             
+	Tcl_DStringAppend(&name, LangString(LangScalarResult(textPtr->interp)), -1);
 	Tcl_ResetResult(textPtr->interp);
 	ewPtr->body.ew.tkwin = Tk_NameToWindow(textPtr->interp,
 		Tcl_DStringValue(&name), textPtr->tkwin);
+	Tcl_DStringFree(&name);
 	if (ewPtr->body.ew.tkwin == NULL) {
 	    goto createError;
 	}
@@ -808,7 +805,7 @@ EmbWinLayoutProc(textPtr, indexPtr, ewPtr, offset, maxX, maxChars,
 		Tcl_AppendResult(textPtr->interp, "can't embed ",
 			Tk_PathName(ewPtr->body.ew.tkwin), " relative to ",
 			Tk_PathName(textPtr->tkwin),          NULL);
-		Tk_BackgroundError(textPtr->interp);
+		Tcl_BackgroundError(textPtr->interp);
 		ewPtr->body.ew.tkwin = NULL;
 		goto gotWindow;
 	    }
@@ -1032,7 +1029,7 @@ EmbWinUndisplayProc(textPtr, chunkPtr)
 	 */
 
 	ewPtr->body.ew.displayed = 0;
-	Tk_DoWhenIdle(EmbWinDelayedUnmap, (ClientData) ewPtr);
+	Tcl_DoWhenIdle(EmbWinDelayedUnmap, (ClientData) ewPtr);
     }
 }
 
